@@ -75,6 +75,8 @@
 #define NOVALUESqual    24
 #define ABOUTqual	25
 #define TOLERANCEqual   26
+#define FORMATqual      27
+#define NOFORMATqual    28
 
 /******************************************************************************
 * Macro constats.
@@ -129,6 +131,7 @@ Logical ParseTolerances PROTOARGs((
 ));
 Logical ValuesCmp PROTOARGs((void *, void *, long, double, double));
 int CheckNaN PROTOARGs((void *, void *, long));
+Logical CompareStrings PROTOARGs((char *, char *, long, long));
 
 /*****************************************************************************/
 
