@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                          CDF software distribution configuration.
 *
@@ -63,13 +68,7 @@
 #define SUPPORT_HUFF64          1
 #define SUPPORT_AHUFF64         1
 #define SUPPORT_GZIP64          1
-/*
-#if defined(dos)
-#  define SUPPORT_ZLIB64        0
-#else
-#  define SUPPORT_ZLIB64        1
-#endif
-*/
+
 /******************************************************************************
 * Supported checksums.  1 = yes, 0 = no.
 ******************************************************************************/
@@ -143,7 +142,7 @@
 /*****************************************************************************
 * Blocking factors.
 *****************************************************************************/
-#if defined(win32) && (_FILE_OFFSET_BITS == 64)
+#if defined(win32)
 
 #  define MIN_BLOCKING_BYTES_standard     512
 #  define MIN_BLOCKING_RECS_standard      1
@@ -151,17 +150,28 @@
 #  define MIN_BLOCKING_BYTES_sparse       2048
 #  define MIN_BLOCKING_RECS_sparse        1
 
-#  define MIN_BLOCKING_BYTES_compressed   5120
+#  define MIN_BLOCKING_BYTES_compressed   65536
 #  define MIN_BLOCKING_RECS_compressed    1
 #else
-#  define MIN_BLOCKING_BYTES_standard     512
-#  define MIN_BLOCKING_RECS_standard      1
+#  if defined(unix) || defined(__i386) || defined(__x86_64)
+#    define MIN_BLOCKING_BYTES_standard     8192
+#    define MIN_BLOCKING_RECS_standard      1
 
-#  define MIN_BLOCKING_BYTES_sparse       2048
-#  define MIN_BLOCKING_RECS_sparse        1
+#    define MIN_BLOCKING_BYTES_sparse       16384
+#    define MIN_BLOCKING_RECS_sparse        1
 
-#  define MIN_BLOCKING_BYTES_compressed   5120
-#  define MIN_BLOCKING_RECS_compressed    1
+#    define MIN_BLOCKING_BYTES_compressed   65536
+#    define MIN_BLOCKING_RECS_compressed    1
+#  else
+#    define MIN_BLOCKING_BYTES_standard     1024
+#    define MIN_BLOCKING_RECS_standard      1
+
+#    define MIN_BLOCKING_BYTES_sparse       4096
+#    define MIN_BLOCKING_RECS_sparse        1
+
+#    define MIN_BLOCKING_BYTES_compressed   65536
+#    define MIN_BLOCKING_RECS_compressed    1
+#  endif
 #endif
 /*****************************************************************************
 * Should trailing blanks be automatically stripped from CDF pathnames when
@@ -186,10 +196,10 @@
 
 /*****************************************************************************
 * Should the default encoding for CDFs created with the Internal Interface be
-* HOST_ENCODING (the alternative is NETWORK_ENCODING)?
+* HOST_ENCODING (the alternative is IBMPC_ENCODING)?
 *****************************************************************************/
 
-#define DEFAULT_TO_HOST_ENCODING        TRUE    /* TRUE = yes, FALSE = no. */
+#define DEFAULT_TO_HOST_ENCODING        TRUE     /* TRUE = yes, FALSE = no. */
 
 /*****************************************************************************
 * Should the default majority for CDFs created with the Internal Interface be
@@ -231,21 +241,24 @@
 * The default pad values for each data type.
 *****************************************************************************/
 
-#define DEFAULT_BYTE_PADVALUE           0
-#define DEFAULT_INT1_PADVALUE           0
-#define DEFAULT_UINT1_PADVALUE          0
-#define DEFAULT_INT2_PADVALUE           0
-#define DEFAULT_UINT2_PADVALUE          0
-#define DEFAULT_INT4_PADVALUE           0
-#define DEFAULT_UINT4_PADVALUE          0
-#define DEFAULT_REAL4_PADVALUE          0.0
-#define DEFAULT_FLOAT_PADVALUE          0.0
-#define DEFAULT_REAL8_PADVALUE          0.0
-#define DEFAULT_DOUBLE_PADVALUE         0.0
+#define DEFAULT_BYTE_PADVALUE           (-127)
+#define DEFAULT_INT1_PADVALUE           (-127)
+#define DEFAULT_UINT1_PADVALUE          ((unsigned char)254)
+#define DEFAULT_INT2_PADVALUE           ((short)-32767)
+#define DEFAULT_UINT2_PADVALUE          ((unsigned short)65534)
+#define DEFAULT_INT4_PADVALUE           (-2147483647L)
+#define DEFAULT_INT8_PADVALUE           ((long long)-9223372036854775807LL)
+#define DEFAULT_UINT4_PADVALUE          (4294967294U)
+#define DEFAULT_REAL4_PADVALUE          ((float)-1.0E30f)
+#define DEFAULT_FLOAT_PADVALUE          ((float)-1.0E30f)
+#define DEFAULT_REAL8_PADVALUE          -1.0E30
+#define DEFAULT_DOUBLE_PADVALUE         -1.0E30
 #define DEFAULT_CHAR_PADVALUE           ' '
 #define DEFAULT_UCHAR_PADVALUE          ' '
 #define DEFAULT_EPOCH_PADVALUE          0.0
-
+#define DEFAULT_EPOCH16_PADVALUE        0.0
+#define DEFAULT_TT2000_PADVALUE         ((long long)-9223372036854775807LL)
+                                        /* 0000-01-01T00:00:00.000000000 */
 /*****************************************************************************
 * By default, convert -0.0 to 0.0 when read from or written to a CDF by the
 * CDF library?  (Values already existing in the CDF are not modified.)  TRUE
@@ -303,11 +316,12 @@
 #define DEFAULTheadingEXPORT		TRUE	/* [no]heading */
 #define DEFAULTsingleEXPORT             TRUE    /* TRUE = "single",
 						   FALSE = "multi". */
-#define DEFAULTnetworkEXPORT            TRUE    /* TRUE = "network",
+#define DEFAULTnetworkEXPORT            FALSE   /* TRUE = "network",
 						   FALSE = "host". */
 #define DEFAULTepochEXPORT              0       /* 0 (standard), 1, 2, 3, 4
-						   (C/Fortran format), or 5
-						   (custom format). */
+						   (C/Fortran format), 5
+						   (custom format), or 6
+                                                   (ISO 8601). */
 #define DEFAULThorizontalEXPORT         FALSE   /* TRUE = "horizontal",
 						   FALSE = "vertical". */
 #define DEFAULTmajorityEXPORT           0       /* ROW_MAJOR, COLUMN_MAJOR, or
@@ -331,6 +345,8 @@
 #define DEFAULTspacingSIMPLE		1
 #define DEFAULTheadingSIMPLE		TRUE
 #define DEFAULTzModeSIMPLE		zMODEon2
+#define DEFAULTcdaweb			FALSE
+#define DEFAULTvirtual                  FALSE
 
 /******************************************************************************
 * Command line defaults for CDFedit.
@@ -387,11 +403,12 @@
 #define DEFAULTsizesIRsDUMP        FALSE        /* [no]sizes */
 #define DEFAULTindexingIRsDUMP     FALSE        /* [no]indexing */
 #define DEFAULTsummaryIRsDUMP      TRUE         /* [no]summary */
-#define DEFAULTradixIRsDUMP	10		/* 10 = decimal (file offsets),
+#define DEFAULTradixIRsDUMP	   10		/* 10 = decimal (file offsets),
 						   16 = hexadecimal. */
 #define DEFAULTlevelIRsDUMP        1            /* 1 == "brief",
 						   2 == "most",
 						   3 == "full" */
+#define DEFAULTdataIRsDUMP	   FALSE	/* data. */
 
 /******************************************************************************
 * Command line defaults for CDFcompare.
@@ -409,6 +426,7 @@
 #define DEFAULTetcCMP           TRUE            /* [no]etc */
 #define DEFAULTlocationCMP      FALSE           /* [no]location */
 #define DEFAULTvalueCMP         FALSE           /* [no]value */
+#define DEFAULTformatCMP        TRUE            /* [no]format */
 #define DEFAULTstatsCMP         FALSE           /* [no]statistics */
 #define DEFAULTzModeCMP         zMODEoff        /* zMODEoff = zMode/0,
 						   zMODEon1 = zMode/1,
@@ -469,6 +487,7 @@
 #define DEFAULTdeleteSKT2CDF    FALSE           /* [no]delete */
 #define DEFAULTfillvalSKT2CDF   FALSE           /* [no]fillval */
 #define DEFAULTstatsSKT2CDF	FALSE		/* [no]statistics */
+#define DEFAULTwarningSKT2CDF	TRUE		/* [no]warning */
 #define DEFAULTzModeSKT2CDF     zMODEoff        /* zMODEoff = zMode/0,
 						   zMODEon1 = zMode/1,
 						   zMODEon2 = zMode/2 */
@@ -480,9 +499,10 @@
 * means `noxxxxx'.
 ******************************************************************************/
 
-#define DEFAULTshortDUMP        FALSE           /* [no]short */
 #define DEFAULTformatDUMP       TRUE            /* [no]format */
-#define DEFAULTnoheaderDUMP     FALSE           /* noheader */
+#define DEFAULTheaderDUMP       TRUE            /* [no]header */
+#define DEFAULTcol2rowDUMP      FALSE           /* col2row    */
+#define DEFAULTepochVVDUMP      FALSE           /* epochVV    */
 
 /******************************************************************************
 * Command line defaults for CDFmerge.
@@ -496,6 +516,15 @@
 #define DEFAULTcdawebMERGE      FALSE           /* [no]cdaweb */
 #define DEFAULTaugmentlabelMERGE   FALSE        /* [no]augment_label */
 
+/******************************************************************************
+* Command line defaults for CDFleapsecondsinfo.
+* These defaults can be overridden on the command line when CDFleaptableinfo
+* is executed.  
+******************************************************************************/
+
+#define DEFAULTdumpLEAPTABLE    FALSE           /* TRUE if /DUMP, -dump 
+                                                   specified. */
+
 /******************************************************************************
 * `Setup' menu defaults for CDFlist.
 * These defaults can be overridden on the command line when CDFlist is
@@ -509,7 +538,7 @@
 #define DEFAULTeraseLIST        TRUE            /* [no]erase */
 #define DEFAULTsingleLIST       TRUE            /* TRUE == "single",
 						   FALSE == "multi". */
-#define DEFAULTnetworkLIST      TRUE            /* TRUE == "network",
+#define DEFAULTnetworkLIST      FALSE           /* TRUE == "network",
 						   FALSE == "host". */
 #define DEFAULTepochLIST        TRUE            /* TRUE == "epoch",
 						   FALSE == "epoch1". */
@@ -527,6 +556,17 @@
 
 #define DEFAULTformatWALK       TRUE            /* [no]format */
 
+/******************************************************************************
+* `Setup' menu defaults for CDFvalidate.
+* These defaults can be overridden on the command line when CDFvalidate is
+* executed.  For the [no]xxxxx-style qualifiers TRUE means `xxxxx' and FALSE
+* means `noxxxxx'.
+******************************************************************************/
+
+#define DEFAULTvalidateValidate TRUE            /* [no]validate */
+#define DEFAULTdebugValidate    FALSE           /* [no]debug */
+#define DEFAULTquietValidate    TRUE            /* [no]quiet */
+
 /******************************************************************************
 * Key definitions for the toolkit program using a full-screen interface.
 * These are currently CDFedit, CDFexport, CDFlist, and CDFwalk.  The key
