@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                            CDFexport header file.
 *
@@ -250,7 +255,7 @@
 #define SAVEkey_SIMPLE                  KB_CTRL_J
 #define RESTOREkey_SIMPLE               KB_CTRL_L
 
-#if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
+#if defined(alphaosf) || defined(IRIX64bit) || defined(_LP64) || \
     defined(__amd64) || defined(__x86_64__) || defined(__ia64__) || \
     defined(__PPC64__) || defined(__ppc64__)
 #  define Int32FORMATstandard     "%11d"
@@ -381,7 +386,6 @@
 #define HUFF0in                 2
 #define AHUFF0in                3
 #define GZIPin                  4
-/* #define ZLIB0in                 5 */
 
 #define NOsRECORDSin            0
 #define PADsRECORDSin           1
@@ -664,7 +668,7 @@ struct VarStruct {
   void *fill;           /* FILLVAL entry converted to variable's data type. */
   void *pad;            /* Pad value. */
   void *value;          /* Buffer for one value. */
-  Byte *buffer;         /* Buffer for many values. */
+  Byte1 *buffer;        /* Buffer for many values. */
   char *name;           /* Variable name. */
   char *format;         /* Format (C or Fortran). */
   size_t nValueBytes;   /* Number of bytes per variable value. */
@@ -681,7 +685,7 @@ struct VarStruct {
 ******************************************************************************/
 
 struct ItemStruct {
-  int type;                     /* RECORDSt, INDICESt, or VARIABLEt. */
+  int type;                     /* RECORDt, INDICESt, or VARIABLEt. */
   int width;
   Logical outputSetting;        /* Setting on the SelectionWindow. */
   Logical filterSetting;        /* Setting on the SelectionWindow. */
@@ -953,7 +957,7 @@ int OutputNRVvalues PROTOARGs((
 Logical OutputHyperBuffer PROTOARGs((
   CDFid outID, long varNo, Logical zVar, long outMajority, long recNumber,
   long recCount, long dimIndices[], long dimCounts[], long numDims,
-  long dimSizes[], Byte *buffer, long nValues, Logical fullRecord,
+  long dimSizes[], Byte1 *buffer, long nValues, Logical fullRecord,
   size_t nValueBytes, long nRecordValues
 ));
 int CopyAttributesANDgEntries PROTOARGs((
@@ -1013,7 +1017,7 @@ long FindLastRecord PROTOARGs((
   long recCount
 ));
 void FilterBuffer PROTOARGs((
-  struct ItemStruct *Item, Byte *buffer, long nValues
+  struct ItemStruct *Item, Byte1 *buffer, long nValues
 ));
 Logical AbortListing PROTOARGs((FILE *oFp, char *line));
 Logical AbortCDF PROTOARGs((struct ItemStruct *exportHead));
@@ -1028,11 +1032,11 @@ Logical IsaVirtualVariable PROTOARGs((Logical zVAR, char *varName));
 CDFstatus GetVirtualVariableFunc PROTOARGs((Logical zVAR, char *varName, 
 char *funcName));
 CDFstatus ReadRealorVirtualVariableEpoch PROTOARGs((Logical zVAR,
-char *varName, int ind, double **epochValues));
+char *varName, int ind, void **epochValues));
 CDFstatus ReadVirtualVariableApplyQaflag PROTOARGs((Logical zVar, char *varName,
 int *qaFlags));
 int FindUsedEpochs PROTOARGs((struct ItemStruct **itemHead, int *index));
-CDFstatus LoadVirtualEpochs PROTOARGs((double **epochValues));
+CDFstatus LoadVirtualEpochs PROTOARGs((void **epochValues));
 #if defined(mac)
 Logical ExportQOPs PROTOARGs((
   int *argC, char **argV[]
