@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                    Header file for `windows'.
 *
@@ -165,7 +170,7 @@ typedef int chtype;
 #  if defined(ultrix) || defined(AIX) || defined(hpux)
 #    define problemACS
 #  endif
-#  if defined(alphaosf) || defined(IRIX64bit) || defined(SOLARIS64) || \
+#  if defined(alphaosf) || defined(IRIX64bit) || defined(_LP64) || \
       defined(posixSHELL) || defined(__QNX__) || \
       defined(__CYGWIN__) || defined(__MINGW32__)
 #    define problemACS
@@ -297,8 +302,12 @@ typedef int chtype;
 #  define CYGWIN_DELETE         263
 #endif
 
+#if defined(linux)
+#  define LINUX_DELETE          263
+#endif
+
 #if defined(__MINGW32__)
-#  define MINGW32_DELETE         263
+#  define MINGW32_DELETE        263
 #endif
 
 /******************************************************************************
@@ -364,7 +373,7 @@ typedef struct COWvdStruct {	/* COW "virtual display" structure. */
 #endif
 
 #if defined(SMGui)
-  typedef uLong LocalId;		/* SMG virtual display identifier. */
+  typedef uLongx LocalId;		/* SMG virtual display identifier. */
 #endif
 #if defined(CURSESui)
   typedef WINDOW *LocalId;		/* Pointer to Curses window. */
