@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the 
+* Administrator of the National Aeronautics and Space Administration. 
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                CDF library allocation functions.
 *
@@ -8,6 +13,8 @@
 *
 *   V1.0  22-Jul-96, J Love     Original version.
 *   V2.0  29-Jun-04, M Liu      Added LFS (Large File Support > 2G).
+*   V3.0   7-May-10, M Liu      Modified to update file's length64 when a 
+*                               VVR/CVVR is allocated, if needed. 
 *
 ******************************************************************************/
 
@@ -854,6 +861,8 @@ OFF_T *vvrOffset;
   if (!sX(WriteVVR64(CDF->fp,*vvrOffset,
 		     VVR_RECORDx,&VVR,
 		     VVR_NULL),&pStatus)) return pStatus;
+  CDF->fp->length64 = MaxLongLong (CDF->fp->length64, *vvrOffset + 
+                                                      VVR.RecordSize);
   return pStatus;
 }
 
@@ -876,6 +885,8 @@ OFF_T *cvvrOffset;
   if (!sX(WriteCVVR64(CDF->fp,*cvvrOffset,
 		      CVVR_RECORDx,&CVVR,
 		      CVVR_NULL),&pStatus)) return pStatus;
+  CDF->fp->length64 = MaxLongLong (CDF->fp->length64, *cvvrOffset + 
+                                                      CVVR.RecordSize);
   return pStatus;
 }
 
