@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                           CDF `close' operations.
 *
@@ -217,6 +222,7 @@ struct CurStruct *Cur;
         /**********************************************************************
 	* Not compressed - first close the dotCDF file.
 	**********************************************************************/
+	if (CDF->readOnly) ResetReadOnlyState(CDF);
 	if (!CLOSEv(CDF->dotFp,CDF,&vStats)) {
 	  CDF->dotFp = NULL;
 	  AbortAccess (CDF, noUPDATE, noDELETE);
