@@ -41,7 +41,13 @@
 *                               a set of similar functions for put operations.
 *   V3.5  28-May-12, M. Liu     Added CDFinsertVarRecordsByVarID and
 *                               CDFinsertVarRecordsByVarName.
-*
+*   V3.6  23-Sep-16, M. Liu     Added functions: CDFreadzVarAllByVarID,
+*                               CDFreadzVarAllByVarName, CDFreadzVarPadValue,
+*                               CDFreadzVarDataByVarID,
+*                               CDFreadzVarDataByVarName,
+*                               CDFreadzVarRangeDataByVarID,
+*                               CDFreadzVarRangeDataByVarName, CDFgetzVarSpec,
+*                               CDFreadgAttrEntry and CDFreadzAttrEntry.
 ******************************************************************************/
 
 #include "cdflib.h"
@@ -242,6 +248,102 @@ void    *value;         /* Out -- Value. */
   return pStatus;
 }
 
+/******************************************************************************
+* CDFdataFree.
+* Free the dynamic space if it exists.It applies to pointer of CDFdata type.
+******************************************************************************/
+
+VISIBLE_PREFIX void CDFdataFree (data)
+CDFdata data;           /* In -- data buffer. */
+{
+  if (data != NULL) free (data);
+}
+
+/******************************************************************************
+* CDFreadgAttrEntry.
+* This function reads a global attribute entry. This function passes back the
+* pointer to the dynamically allocated space for the retrieved data.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadgAttrEntry (id, attrNum, entryNum, dataType,
+                                            numElems, value)
+CDFid   id;             /* In -- CDF id. */
+long    attrNum;        /* In -- gAttribute number. */
+long    entryNum;       /* In -- gEntry number. */
+long    *dataType;      /* Out -- Entry's data type. */
+long    *numElems;      /* Out -- Entry's number of elements. */
+CDFdata *value;         /* Out -- Entry's value. */
+{
+  long scope;
+  CDFstatus pStatus = CDF_OK;
+  void *buf;
+
+  *value = (CDFdata) NULL;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          ATTR_, attrNum,
+                 GET_, ATTR_SCOPE_, &scope,
+                 NULL_), &pStatus)) return pStatus;
+  if (!GLOBALscope(scope)) return ILLEGAL_FOR_SCOPE;
+  if (!sX(CDFlib(SELECT_, gENTRY_, entryNum,
+                 GET_, gENTRY_DATATYPE_, dataType,
+                       gENTRY_NUMELEMS_, numElems,
+                 NULL_), &pStatus)) return pStatus;
+  buf = (void *) malloc ((size_t) CDFelemSize(*dataType) *
+                         (STRINGdataType(*dataType)?(*numElems+1):*numElems));
+  if (buf == NULL) return BAD_MALLOC;
+  if (!sX(CDFlib(GET_, gENTRY_DATA_, buf,
+                 NULL_), &pStatus)) return pStatus;
+  if (STRINGdataType (*dataType)) *((char *) buf + *numElems) = '\0';
+  *value = (CDFdata) buf;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzAttrEntry.
+* This function reads a zVariable attribute entry. This function passes back
+* the pointer to the dynamically allocated space for the retrieved data.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzAttrEntry (id, attrNum, entryNum, dataType,
+                                            numElems, value)
+CDFid   id;             /* In -- CDF id. */
+long    attrNum;        /* In -- Attribute number. */
+long    entryNum;       /* In -- zEntry number. */
+long    *dataType;      /* Out -- Entry's data type. */
+long    *numElems;      /* Out -- Entry's number of elements. */
+CDFdata *value;         /* Out -- Entry's value. */
+{
+  long scope, dataTypeX;
+  CDFstatus pStatus = CDF_OK;
+  void *buf;
+
+  *value = (CDFdata) NULL;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          ATTR_, attrNum,
+                 GET_, ATTR_SCOPE_, &scope,
+                 NULL_), &pStatus)) return pStatus;
+  if (GLOBALscope(scope)) return ILLEGAL_FOR_SCOPE;
+  pStatus = CDFlib(SELECT_, CDF_, id,
+                            zVAR_, entryNum,
+                   GET_, zVAR_DATATYPE_, &dataTypeX,
+                   NULL_);
+  if (pStatus != CDF_OK) return NO_SUCH_VAR;
+  if (!sX(CDFlib(SELECT_, zENTRY_, entryNum,
+                 GET_, zENTRY_DATATYPE_, dataType,
+                       zENTRY_NUMELEMS_, numElems,
+                 NULL_), &pStatus)) return pStatus;
+  buf = (void *) malloc ((size_t) CDFelemSize(*dataType) *
+                         (STRINGdataType(*dataType)?(*numElems+1):*numElems));
+  if (buf == NULL) return BAD_MALLOC;
+  if (!sX(CDFlib(GET_, zENTRY_DATA_, buf,
+                 NULL_), &pStatus)) return pStatus;
+  if (STRINGdataType (*dataType)) *((char *) buf + *numElems) = '\0';
+  *value = (CDFdata) buf;
+  return pStatus;
+}
+
 /******************************************************************************
 * CDFdeleteAttrEntry.
 * Can't implement with macro because the attribute's scope determines which
@@ -505,9 +607,9 @@ void    *buffer;        /* Out -- Buffer for holding data. */
   varNums[0] = varNum;
 
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
                           recNum,
-                 GET_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 GET_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
@@ -533,9 +635,9 @@ void    *buffer;        /* In -- Buffer for holding data. */
   varNums[0] = varNum;
 
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_),
                           recNum,
-                 PUT_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 PUT_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
@@ -575,19 +677,19 @@ void    *buffer;        /* Out -- Buffer for holding data. */
     intervals[i] = 1L;
   }
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           lastRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -617,6 +719,341 @@ void    *buffer;        /* Out -- Buffer for holding data. */
   return pStatus;
 }
 
+/******************************************************************************
+* CDFgetzVarSpec.  
+* Acquire a given zVariable's (by zVar id) specification.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFgetzVarSpec (id, varNum, dataType, numElems,
+                                         numDims, dimSizes, recVary, dimVarys)
+CDFid   id;             /* In -- CDF id. */
+long    varNum;         /* In -- Variable number. */
+long    *dataType;      /* Out -- Variables' data type. */
+long    *numElems;      /* Out -- Variable's number of elements. */
+long    *numDims;       /* Out -- Variable's number of dimensions. */
+long    *dimSizes;      /* Out -- Variable's dimensional sizes. */
+long    *recVary;       /* Out -- Variable's record vary. */
+long    *dimVarys;      /* Out -- Variable's dimensional vary. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+
+  if (!sX(CDFgetzVarDataType(id,varNum,dataType), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumElements(id,varNum,numElems), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumDims(id,varNum,numDims), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarRecVariance(id,varNum,recVary), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimVariances(id,varNum,dimVarys), &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarAllByVarID.  
+* Acquire data from all records for a given zVariable (by zVar id).
+* Retrieved data are filled into the buffer. For sparse record variables,
+* virtual records will be filled with specified value. This function passes
+* back the pointer to the dynamically allocated space for the retrieved data.
+* The variable's spec is also returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarAllByVarID (id, varNum, numRecs, dataType,
+                                                numElems, numDims, dimSizes,
+                                                recVary, dimVarys, buffer)
+CDFid   id;             /* In -- CDF id. */
+long    varNum;         /* In -- Variable number. */
+long    *numRecs;       /* Out -- Variable number of written records. */
+long    *dataType;      /* Out -- Variables' data type. */
+long    *numElems;      /* Out -- Variable's number of elements. */
+long    *numDims;       /* Out -- Variable's number of dimensions. */
+long    *dimSizes;      /* Out -- Variable's dimensional sizes. */
+long    *recVary;       /* Out -- Variable's record vary. */
+long    *dimVarys;      /* Out -- Variable's dimensional vary. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  int  i;
+  long lastRec, totalValues;
+  long indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS];
+  void *buf;
+
+  *buffer = (CDFdata) NULL;
+  *numRecs = 0L;
+  if (!sX(CDFgetzVarDataType(id,varNum,dataType), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumElements(id,varNum,numElems), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumDims(id,varNum,numDims), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarRecVariance(id,varNum,recVary), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimVariances(id,varNum,dimVarys), &pStatus)) return pStatus;
+  if (!sX(CDFgetVarMaxWrittenRecNum(id,1,varNum,&lastRec), &pStatus))
+    return pStatus;
+  if (lastRec == -1) return CDF_OK;
+  *numRecs = lastRec + 1;
+  totalValues = *numRecs;
+  for (i = 0; i < (int) *numDims; ++i) {
+    indices[i] = 0L;
+    intervals[i] = 1L;
+    totalValues *= dimSizes[i];
+  }
+  buf = (void *) malloc ((size_t) CDFelemSize (*dataType) * totalValues *
+                         *numElems);
+  if (buf == NULL) return BAD_MALLOC;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          zVAR_, varNum,
+                          zVAR_RECNUMBER_, 0L,
+                          zVAR_RECCOUNT_, *numRecs,
+                          zVAR_RECINTERVAL_, 1L,
+                          zVAR_DIMINDICES_, indices,
+                          zVAR_DIMCOUNTS_, dimSizes,
+                          zVAR_DIMINTERVALS_, intervals,
+                 GET_, zVAR_HYPERDATA_, buf,
+                 NULL_), &pStatus)) return pStatus;
+  *buffer = (CDFdata) buf;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarDataByVarID.
+* Acquire data from all records for a given zVariable (by zVar id).
+* Retrieved data are filled into the buffer. For sparse record variables,
+* virtual records will be filled with specified value. This function passes
+* back the pointer to the dynamically allocated space for the retrieved data.
+* No variable spec is returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarDataByVarID (id, varNum, numRecs, buffer)
+CDFid   id;             /* In -- CDF id. */
+long    varNum;         /* In -- Variable number. */
+long    *numRecs;       /* Out -- Number of records read into buffer. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  int  i;
+  long dataType, numElems, numDims, lastRec, totalValues;
+  long dimSizes[CDF_MAX_DIMS], indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS];
+  void *buf;
+
+  *buffer = (CDFdata) NULL;
+  *numRecs = 0L;
+  if (!sX(CDFgetzVarDataType(id,varNum,&dataType), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumElements(id,varNum,&numElems), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
+  if (!sX(CDFgetVarMaxWrittenRecNum(id,1,varNum,&lastRec), &pStatus))
+    return pStatus;
+  if (lastRec == -1) return CDF_OK;
+  totalValues = lastRec + 1L;
+  for (i = 0; i < (int) numDims; ++i) {
+    indices[i] = 0L;
+    intervals[i] = 1L;
+    totalValues *= dimSizes[i];
+  }
+  buf = (void *) malloc ((size_t) CDFelemSize (dataType) * totalValues *
+                         numElems);
+  if (buf == NULL) return BAD_MALLOC;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          zVAR_, varNum,
+                          zVAR_RECNUMBER_, 0L,
+                          zVAR_RECCOUNT_, (lastRec + 1L),
+                          zVAR_RECINTERVAL_, 1L,
+                          zVAR_DIMINDICES_, indices,
+                          zVAR_DIMCOUNTS_, dimSizes,
+                          zVAR_DIMINTERVALS_, intervals,
+                 GET_, zVAR_HYPERDATA_, buf,
+                 NULL_), &pStatus)) return pStatus;
+  *buffer = (CDFdata) buf;
+  *numRecs = lastRec + 1L;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarRangeDataByVarID.  
+* Acquire a range of records from a given zVariable (by zVar id).
+* Retrieved data are filled into the buffer. For a sparse record variable,
+* virtual record will be filled with specified value(s). This function passes
+* back the pointer to the dynamically allocated space for the retrieved data. 
+* No spec from this variable is returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarRangeDataByVarID (id, varNum, strNum,
+                                                      stpNum, buffer)
+CDFid   id;             /* In -- CDF id. */
+long    varNum;         /* In -- Variable number. */
+long    strNum;         /* In -- Variable starting record number. */
+long    stpNum;         /* In -- Variable stopping record number. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  int  i;
+  long dataType, numElems, numDims, totalValues;
+  long dimSizes[CDF_MAX_DIMS], indices[CDF_MAX_DIMS], intervals[CDF_MAX_DIMS];
+  void *buf;
+
+  *buffer = (CDFdata) NULL;
+  if (strNum > stpNum || stpNum < 0) return CANNOT_ALLOCATE_RECORDS;
+  if (!sX(CDFgetzVarDataType(id,varNum,&dataType), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumElements(id,varNum,&numElems), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarNumDims(id,varNum,&numDims), &pStatus)) return pStatus;
+  if (!sX(CDFgetzVarDimSizes(id,varNum,dimSizes), &pStatus)) return pStatus;
+  totalValues = stpNum - strNum + 1L;
+  for (i = 0; i < (int) numDims; ++i) {
+    indices[i] = 0L;
+    intervals[i] = 1L;
+    totalValues *= dimSizes[i];
+  }
+  buf = (void *) malloc ((size_t) CDFelemSize (dataType) * totalValues *
+                         numElems);
+  if (buf == NULL) return BAD_MALLOC;
+  if (!sX(CDFlib(SELECT_, CDF_, id,
+                          zVAR_, varNum,
+                          zVAR_RECNUMBER_, strNum,
+                          zVAR_RECCOUNT_, (stpNum-strNum+1L),
+                          zVAR_RECINTERVAL_, 1L,
+                          zVAR_DIMINDICES_, indices,
+                          zVAR_DIMCOUNTS_, dimSizes,
+                          zVAR_DIMINTERVALS_, intervals,
+                 GET_, zVAR_HYPERDATA_, buf,
+                 NULL_), &pStatus)) return pStatus;
+  *buffer = (CDFdata) buf;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarAllByVarName.  
+* Acquire data from all records for a given variable (by its unique name).
+* Retrieved data are filled into the buffer. For sparse record variables,
+* virtual records will be filled with specified value. This function passes
+* back the pointer to the dynamically allocated space for the retrieved data.
+* The variable's spec is also returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarAllByVarName (id, varName, numRecs,
+                                                  dataType, numElems, numDims,
+                                                  dimSizes, recVary, dimVarys,
+                                                  buffer)
+CDFid   id;             /* In -- CDF id. */
+char    *varName;       /* In -- Variable name. */
+long    *numRecs;       /* Out -- Variable number of written records. */
+long    *dataType;      /* Out -- Variables' data type. */
+long    *numElems;      /* Out -- Variable's number of elements. */
+long    *numDims;       /* Out -- Variable's number of dimensions. */
+long    *dimSizes;      /* Out -- Variable's dimensional sizes. */
+long    *recVary;       /* Out -- Variable's record vary. */
+long    *dimVarys;      /* Out -- Variable's dimensional vary. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  long varNum;
+
+  *numRecs = 0L;
+  *buffer = (CDFdata) NULL;
+  varNum = CDFgetVarNum(id, varName);
+  if (varNum < 0) return NO_SUCH_VAR;
+  if (!sX(CDFreadzVarAllByVarID(id,varNum,numRecs,dataType,numElems,numDims,
+                                dimSizes,recVary,
+                                     dimVarys,buffer),
+          &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarDataByVarName.  
+* Acquire data from all records for a given variable (by its unique name).
+* Retrieved data are filled into the buffer. For sparse record variables,
+* virtual records will be filled with specified value. This function passes
+* back the pointer to the dynamically allocated space for the retrieved data.
+* No spec from the variable is returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarDataByVarName (id, varName, numRecs,
+                                                   buffer)
+CDFid   id;             /* In -- CDF id. */
+char    *varName;       /* In -- Variable name. */
+long    *numRecs;       /* Out -- Number of records read into the buffer. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  long varNum;
+
+  *buffer = (CDFdata) NULL;
+  *numRecs = 0;
+  varNum = CDFgetVarNum(id, varName);
+  if (varNum < 0) return NO_SUCH_VAR;
+  if (!sX(CDFreadzVarDataByVarID(id,varNum,numRecs,buffer),
+          &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarRangeDataByVarName.  
+* Acquire a range of records for a given variable (by its unique name).
+* Retrieved data are filled into the buffer. For a sparse record variable,
+* virtual records will be filled with specified value(s). This function passes
+* back the pointer to the dynamically allocated space for the retrieved data.
+* The variable's spec is also returned with this function.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarRangeDataByVarName (id, varName, strNum,
+                                                        stpNum, buffer)
+CDFid   id;             /* In -- CDF id. */
+char    *varName;       /* In -- Variable name. */
+long    strNum;         /* In -- Variable starting record number. */
+long    stpNum;         /* In -- Variable stopping record number. */
+CDFdata *buffer;        /* Out -- Buffer for holding data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  long varNum;
+
+  *buffer = (CDFdata) NULL;
+  varNum = CDFgetVarNum(id, varName);
+  if (varNum < 0) return NO_SUCH_VAR;
+  if (!sX(CDFreadzVarRangeDataByVarID(id,varNum,strNum,stpNum,buffer),
+          &pStatus)) return pStatus;
+  return pStatus;
+}
+
+/******************************************************************************
+* CDFreadzVarPadValue.  
+* Acquire the pad data for a given variable (by its variable id).
+* The status will return NO_PADVALUE_SPECIFIED (1005) if the pad value is not
+* set. Otherwise, CDF_OK (0) is returned.
+* <NOTE>: Caller is responsible to free the allocated space.
+******************************************************************************/
+
+VISIBLE_PREFIX CDFstatus CDFreadzVarPadValue (id, varNum, dataType, numElms,
+                                              buffer)
+CDFid   id;             /* In -- CDF id. */
+long    varNum;         /* In -- Variable id. */
+long    *dataType;      /* Out -- Pad value's data type (same as variable's).*/
+long    *numElms;       /* Out -- Pad value's number of elements. */
+CDFdata *buffer;        /* Out -- Buffer for holding the pad data. */
+
+{
+  CDFstatus pStatus = CDF_OK;
+  void *temp;
+
+  *buffer = (CDFdata) NULL;
+  pStatus = CDFgetzVarDataType(id, varNum, dataType);
+  pStatus = CDFgetzVarNumElements(id, varNum, numElms);
+  temp = (void *) malloc (CDFelemSize(*dataType) * *numElms +
+                          (STRINGdataType(*dataType)?1:0));
+  if (!sX(CDFgetzVarPadValue(id,varNum,temp),&pStatus)) return pStatus;
+  if (STRINGdataType(*dataType)) *((char *)temp+*numElms) = '\0';
+  *buffer = (CDFdata) temp;
+  return pStatus;
+}
+
 /******************************************************************************
 * CDFputVarAllRecordsByVarID.
 * Write a numbr of records for a given variable (by zVar flag and id), 
@@ -651,25 +1088,25 @@ void    *buffer;        /* In -- Buffer for holding data. */
   if (!sX(CDFgetVarMaxWrittenRecNum(id,zVar,varNum,&maxRec),&pStatus))
     return pStatus;
   if (!sX(CDFlib(SELECT_, CDF_, id, 
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),0L,
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           numRecs,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   /****************************************************************************
   * Update maximum record numbers.
   ****************************************************************************/
   if (maxRec > (numRecs-1)) {
-    if (!sX(CDFlib(DELETE_, BOO((zVar == 1),zVAR_RECORDS_,rVAR_RECORDS_),
+    if (!sX(CDFlib(DELETE_, BOO((zVar==1),zVAR_RECORDS_,rVAR_RECORDS_),
                             numRecs, maxRec,
                    NULL_), &pStatus)) return pStatus;
   }
@@ -737,20 +1174,20 @@ void    *buffer;        /* Out -- Buffer for holding data. */
     intervals[i] = 1L;    
   }                       
   if (!sX(CDFlib(SELECT_, CDF_, id, 
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                           startRec,
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -817,20 +1254,20 @@ void    *buffer;        /* In -- Buffer for holding data. */
     intervals[i] = 1L;    
   }                       
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                           startRec, 
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           stopRec-startRec+1,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -932,57 +1369,57 @@ void    *buffer;        /* In -- Buffer for holding data. */
         startRecX = startRec;
       }
       if (!sX(CDFlib(SELECT_, CDF_, id,
-                              BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                              BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                              BOO((zVar==1),zVAR_,rVAR_), varNum,
+                              BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                               startRecX, 
-                              BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                              BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                               recs,
-                              BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                              BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                               1L,
-                              BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                              BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                               indices,
-                              BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                              BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                               dimSizes,
-                              BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                              BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                               intervals,
-                     GET_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
+                     GET_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
                      NULL_), &pStatus)) return pStatus;
       startRecY = startRecX + numRecs;
       if (!sX(CDFlib(SELECT_, CDF_, id,
-                              BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                              BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                              BOO((zVar==1),zVAR_,rVAR_), varNum,
+                              BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                               startRecY, 
-                              BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                              BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                               recs,
-                              BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                              BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                               1L,
-                              BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                              BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                               indices,
-                              BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                              BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                               dimSizes,
-                              BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                              BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                               intervals,
-                     PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
+                     PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),tmp,
                      NULL_), &pStatus)) return pStatus;
       
     }
     cdf_FreeMemory (tmp, NULL);
   }
   if (!sX(CDFlib(SELECT_, CDF_, id,
-                          BOO((zVar == 1),zVAR_,rVAR_), varNum,
-                          BOO((zVar == 1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
+                          BOO((zVar==1),zVAR_,rVAR_), varNum,
+                          BOO((zVar==1),zVAR_RECNUMBER_,rVARs_RECNUMBER_),
                           startRec, 
-                          BOO((zVar == 1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
+                          BOO((zVar==1),zVAR_RECCOUNT_,rVARs_RECCOUNT_),
                           numRecs,
-                          BOO((zVar == 1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
+                          BOO((zVar==1),zVAR_RECINTERVAL_,rVARs_RECINTERVAL_),
                           1L,
-                          BOO((zVar == 1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
+                          BOO((zVar==1),zVAR_DIMINDICES_,rVARs_DIMINDICES_),
                           indices,
-                          BOO((zVar == 1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
+                          BOO((zVar==1),zVAR_DIMCOUNTS_,rVARs_DIMCOUNTS_),
                           dimSizes,
-                          BOO((zVar == 1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
+                          BOO((zVar==1),zVAR_DIMINTERVALS_,rVARs_DIMINTERVALS_),
                           intervals,
-                 PUT_, BOO((zVar == 1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
+                 PUT_, BOO((zVar==1),zVAR_HYPERDATA_,rVAR_HYPERDATA_),buffer,
                  NULL_), &pStatus)) return pStatus;
   return pStatus;
 }
@@ -1053,19 +1490,19 @@ void    *buffptr[];     /* Out -- Array of buffer pointers for holding data. */
   varNums = phyRecSize+numVars;
 
   for (i = 0; i < numVars; i++) {
-    if (!sX(CDFlib(GET_, BOO((zVar == 1),zVAR_NUMBER_,rVAR_NUMBER_), 
+    if (!sX(CDFlib(GET_, BOO((zVar==1),zVAR_NUMBER_,rVAR_NUMBER_), 
                          varNames[i], &varNums[i], 
                    NULL_), &pStatus)) return pStatus;
-    if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVAR_,rVAR_), varNums[i],
-                   GET_, BOO((zVar == 1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
+    if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVAR_,rVAR_), varNums[i],
+                   GET_, BOO((zVar==1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
                          &dataType,
-                         BOO((zVar == 1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
+                         BOO((zVar==1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
                          &numElems,
-                         BOO((zVar == 1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
+                         BOO((zVar==1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
                          &numDims,
-                         BOO((zVar == 1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
+                         BOO((zVar==1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
                          dimSizes,
-                         BOO((zVar == 1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
+                         BOO((zVar==1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
                          dimVarys,
                    NULL_), &pStatus)) return pStatus;
     if (!sX(CDFlib(GET_, DATATYPE_SIZE_, dataType, &dataTypeSize,
@@ -1087,9 +1524,9 @@ void    *buffptr[];     /* Out -- Array of buffer pointers for holding data. */
   buffer = (void *) cdf_AllocateMemory ((size_t)totalSize, NULL);
   if (buffer == NULL) return BAD_MALLOC;
 
-  if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
+  if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
                          recNum,
-                 GET_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 GET_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
 
@@ -1140,19 +1577,19 @@ void    *buffptr[];     /* In -- Array of buffer pointers for holding data. */
 
   totalSize = 0;
   for (i = 0; i < numVars; i++) {
-    if (!sX(CDFlib(GET_, BOO((zVar == 1),zVAR_NUMBER_,rVAR_NUMBER_), 
+    if (!sX(CDFlib(GET_, BOO((zVar==1),zVAR_NUMBER_,rVAR_NUMBER_), 
                          varNames[i], &varNums[i],
                    NULL_), &pStatus)) return pStatus;
-    if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVAR_,rVAR_), varNums[i],
-                   GET_, BOO((zVar == 1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
+    if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVAR_,rVAR_), varNums[i],
+                   GET_, BOO((zVar==1),zVAR_DATATYPE_,rVAR_DATATYPE_), 
                          &dataType,
-                         BOO((zVar == 1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
+                         BOO((zVar==1),zVAR_NUMELEMS_,rVAR_NUMELEMS_), 
                          &numElems,
-                         BOO((zVar == 1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
+                         BOO((zVar==1),zVAR_NUMDIMS_,rVARs_NUMDIMS_), 
                          &numDims,
-                         BOO((zVar == 1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
+                         BOO((zVar==1),zVAR_DIMSIZES_,rVARs_DIMSIZES_), 
                          dimSizes,
-                         BOO((zVar == 1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
+                         BOO((zVar==1),zVAR_DIMVARYS_,rVAR_DIMVARYS_), 
                          dimVarys,
                    NULL_), &pStatus)) return pStatus;
     if (!sX(CDFlib(GET_, DATATYPE_SIZE_, dataType, &dataTypeSize,
@@ -1180,9 +1617,9 @@ void    *buffptr[];     /* In -- Array of buffer pointers for holding data. */
     offset += phyRecSize[i];
   }
 
-  if (!sX(CDFlib(SELECT_, BOO((zVar == 1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
+  if (!sX(CDFlib(SELECT_, BOO((zVar==1),zVARs_RECNUMBER_,rVARs_RECNUMBER_), 
                          recNum,
-                 PUT_, BOO((zVar == 1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
+                 PUT_, BOO((zVar==1),zVARs_RECDATA_,rVARs_RECDATA_), numVars,
                        varNums, buffer,
                  NULL_), &pStatus)) return pStatus;
 
@@ -1214,7 +1651,7 @@ long    *maxEntry;      /* Out -- Maximum g/r/zEntry number used. */
                  NULL_), &pStatus)) return pStatus;
   if (GLOBALscope(*scope) && (zEntry == 1)) return ILLEGAL_FOR_SCOPE;
   if (!sX(CDFlib(GET_, ATTR_NAME_, attrName,
-                       BOO((zEntry == 1),ATTR_MAXzENTRY_,
+                       BOO((zEntry==1),ATTR_MAXzENTRY_,
                            (BOO(GLOBALscope(*scope),ATTR_MAXgENTRY_,
                                 ATTR_MAXrENTRY_))), maxEntry,
                  NULL_), &pStatus)) return pStatus;
