@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        CDF `delete' operations.
 *
@@ -21,6 +26,7 @@
 *                               is deleted.
 *   V2.2  08-May-10, M Liu      Reset firstRecInVVR/lastRecInVVR when a
 *                               record(s) gets deleted.
+*   V2.3  09-May-14, M Liu      Renumber the sparse records agter delete.
 *
 ******************************************************************************/
 
@@ -34,7 +40,8 @@ static CDFstatus WasteTree_r PROTOARGs((
   struct CDFstruct *CDF, Int32 vxrOffset
 ));
 static CDFstatus DeleteVarRecords PROTOARGs((
-  struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec
+  struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec,
+  Logical reNumber
 ));
 static CDFstatus DeleteRecords PROTOARGs((
   struct CDFstruct *CDF, struct VarStruct *Var, Int32 firstRec, Int32 lastRec,
@@ -406,14 +413,18 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
-  * rVAR_RECORDS_/zVAR_RECORDS_, 
+  * rVAR_RECORDS_/zVAR_RECORDS_/rVAR_RECORDS_RENUMBER_/zVAR_RECORDS__RENUMBER_
   ****************************************************************************/
   case rVAR_RECORDS_:
-  case zVAR_RECORDS_: {
-    Logical zOp = (Va->item == zVAR_RECORDS_);
+  case zVAR_RECORDS_:
+  case rVAR_RECORDS_RENUMBER_:
+  case zVAR_RECORDS_RENUMBER_: {
+    Logical zOp = (Va->item == zVAR_RECORDS_ ||
+                   Va->item == zVAR_RECORDS_RENUMBER_);
     struct CDFstruct *CDF; struct VarStruct *Var;
     Int32 firstRec = (Int32) va_arg (Va->ap, long);
     Int32 lastRec = (Int32) va_arg (Va->ap, long);
+    Logical reNumber;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -429,7 +440,10 @@ switch (Va->item) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
-    if (!sX(DeleteVarRecords(CDF,Var,firstRec,lastRec),&pStatus)) {
+    if (Va->item == rVAR_RECORDS_RENUMBER_ ||
+        Va->item == zVAR_RECORDS_RENUMBER_) reNumber = TRUE;
+    else reNumber = FALSE;
+    if (!sX(DeleteVarRecords(CDF,Var,firstRec,lastRec,reNumber),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
     }
@@ -713,11 +727,12 @@ Int32 vxrOffset;
 * DeleteVarRecords.
 ******************************************************************************/
 
-static CDFstatus DeleteVarRecords (CDF, Var, firstRec, lastRec)
+static CDFstatus DeleteVarRecords (CDF, Var, firstRec, lastRec, reNumber)
 struct CDFstruct *CDF;
 struct VarStruct *Var;
 Int32 firstRec;
 Int32 lastRec;
+Logical reNumber;
 {
   CDFstatus pStatus = CDF_OK; Int32 deletedTo, recNum; Logical found;
   /****************************************************************************
@@ -728,7 +743,7 @@ Int32 lastRec;
     * Standard variable in a single-file CDF.
     **************************************************************************/
     case STANDARD_: {
-      Int32 vxrHead, aboveRecord, deleteCount; Int32 adjust;
+      Int32 vxrHead, aboveRecord, deleteCount;
       /************************************************************************
       * Verify/adjust the records to be deleted.
       ************************************************************************/
@@ -787,7 +802,9 @@ Int32 lastRec;
     /**************************************************************************
     * Sparse records.
     **************************************************************************/
-    case SPARSE_RECORDS_:
+    case SPARSE_RECORDS_: {
+      Int32 aboveRecord, deleteCount;
+      Int32 vxrHead; Int32 maxRec;
       /************************************************************************
       * Flush staging area first.  This will automatically clear the staging
       * area as well.
@@ -805,44 +822,97 @@ Int32 lastRec;
 	 if (!sX(DeleteRecords(CDF,Var,recNum,
 			       lastRec,&deletedTo),&pStatus)) return pStatus;
       }
-      /************************************************************************
-      * Update the control information.
-      ************************************************************************/
-      if (INCLUSIVE(firstRec,Var->maxWritten,lastRec)) {
-	if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
-			   (Int32)(firstRec - 1),
-			   &(Var->maxWritten),&found),&pStatus)) return
-								 pStatus;
-	if (!found) Var->maxWritten = NO_RECORD;
-      }
-      if (INCLUSIVE(firstRec,Var->maxAllocated,lastRec)) {
-	if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
-			   (Int32)(firstRec - 1),
-			   &(Var->maxAllocated),&found),&pStatus)) return
-								   pStatus;
-	if (!found) Var->maxAllocated = NO_RECORD;
-      }
-      /************************************************************************
-      * Update the maximum record field in the VDR.
-      ************************************************************************/
-      if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
-	if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
-			   (Int32)(firstRec - 1),
-			   &(Var->maxRec),&found),&pStatus)) return pStatus;
-	if (!found) Var->maxRec = NO_RECORD;
-	if (!sX(WriteVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
-			 VDR_MAXREC,&(Var->maxRec),
-			 VDR_NULL),&pStatus)) return pStatus;
+      if (reNumber) {
+        deleteCount = lastRec - firstRec + 1;
+        /**********************************************************************
+        * Update the index entries.
+        **********************************************************************/
+        aboveRecord = firstRec - 1;
+        if (!sX(ReadVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
+                        VDR_VXRHEAD,&vxrHead,
+                        VDR_NULL),&pStatus)) return pStatus;
+        if (!sX(UpdateIndexEntries_r(CDF->fp,vxrHead,aboveRecord,deleteCount),
+                                     &pStatus))
+          return pStatus;
+        /**********************************************************************
+        * Update the control information.
+        **********************************************************************/
+        if (firstRec <= Var->maxWritten) {
+          if (lastRec > Var->maxWritten)
+            Var->maxWritten = firstRec - 1;
+          else
+            Var->maxWritten -= deleteCount;
+        }
+        if (firstRec <= Var->maxAllocated) {
+          if (lastRec > Var->maxAllocated)
+            Var->maxAllocated = firstRec - 1;
+          else
+            Var->maxAllocated -= deleteCount;
+        }
+        /**********************************************************************
+        * Update the maximum record field in the VDR.
+        **********************************************************************/
+        maxRec = Var->maxRec;
+        if (firstRec <= Var->maxRec) {
+          if (lastRec > Var->maxRec)
+            maxRec = firstRec - 1;
+          else
+            maxRec -= deleteCount;
+          if (!sX(WriteVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
+                           VDR_MAXREC,&(maxRec),
+                           VDR_NULL),&pStatus)) return pStatus;
+        }
+        if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
+          if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
+                             (Int32)(firstRec - 1),
+                             &(Var->maxRec),&found),&pStatus)) return pStatus;
+          if (!found) Var->maxRec = NO_RECORD;
+          if (!sX(WriteVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
+                           VDR_MAXREC,&(Var->maxRec),
+                           VDR_NULL),&pStatus)) return pStatus;
+        } else
+          Var->maxRec = maxRec;
+      } else {
+        /**********************************************************************
+        * Update the control information.
+        **********************************************************************/
+        if (INCLUSIVE(firstRec,Var->maxWritten,lastRec)) {
+          if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
+                             (Int32)(firstRec - 1),
+                             &(Var->maxWritten),&found),&pStatus)) return
+                                                                   pStatus;
+          if (!found) Var->maxWritten = NO_RECORD;
+        }
+        if (INCLUSIVE(firstRec,Var->maxAllocated,lastRec)) {
+          if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
+                             (Int32)(firstRec - 1),
+                             &(Var->maxAllocated),&found),&pStatus)) return
+                                                                     pStatus;
+          if (!found) Var->maxAllocated = NO_RECORD;
+        }
+        /**********************************************************************
+        * Update the maximum record field in the VDR.
+        **********************************************************************/
+        if (INCLUSIVE(firstRec,Var->maxRec,lastRec)) {
+          if (!sX(PrevRecord(CDF,Var->VDRoffset,Var->zVar,
+                             (Int32)(firstRec - 1),
+                             &(Var->maxRec),&found),&pStatus)) return pStatus;
+          if (!found) Var->maxRec = NO_RECORD;
+          if (!sX(WriteVDR(CDF,CDF->fp,Var->VDRoffset,Var->zVar,
+                           VDR_MAXREC,&(Var->maxRec),
+                           VDR_NULL),&pStatus)) return pStatus;
+        }
       }
       Var->firstRecInVVR = NO_RECORD;
       Var->lastRecInVVR = NO_RECORD;
       Var->offsetOfVVR = NO_OFFSET;
       break;
+    }
     /**************************************************************************
     * Compressed records.
     **************************************************************************/
     case COMPRESSED_: {
-      Int32 vxrHead, aboveRecord, deleteCount; Int32 adjust;
+      Int32 vxrHead, aboveRecord, deleteCount;
       /************************************************************************
       * Flush/clear staging area first.
       ************************************************************************/
