@@ -544,6 +544,14 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
+  * CDF_LEAPSECONDLASTUPDATED_, 
+  ****************************************************************************/
+  case CDF_LEAPSECONDLASTUPDATED_: {
+    long *lastUpdated = va_arg (Va->ap, long *);
+    *lastUpdated = -1L;
+    break;
+  }
+  /****************************************************************************
   * rVAR_NAME_/zVAR_NAME_
   *    Note that a temporary variable is used when reading the variable name.
   * This is because the caller may have only allocated enough memory for the
@@ -1343,7 +1351,7 @@ switch (Va->item) {
         of the pad value is greater than 16, it causes segmentation fault
         for the IDL CDF interface routine.
     ***********************************************************************/
-    if (dataType == CDF_CHAR ||  dataType == CDF_UCHAR) 
+    if (STRINGdataType(dataType)) 
         numElems = 1;
 
     if (PADvalueBITset(flags)) {
