@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                    CDF `put' operations, part 1.
 *
@@ -169,7 +174,7 @@ switch (Va->item) {
     **************************************************************************/
     if (Var != NULL) {
       Var->NvalueElems = newNumElems;
-      Var->NelemBytes = CDFelemSize ((long)newDataType);
+      Var->NelemBytes = (Int32) CDFelemSize ((long)newDataType);
       Var->NvalueBytes = Var->NvalueElems * Var->NelemBytes;
       Var->NphyRecElems = Var->NphyRecValues * Var->NvalueElems;
       Var->NvirtRecElems = Var->NvirtRecValues * Var->NvalueElems;
@@ -870,7 +875,7 @@ switch (Va->item) {
     **************************************************************************/
     Logical zOp = (Va->item == zVARs_RECDATA_), zVar;
     struct VarStruct *Var; struct CDFstruct *CDF;
-    Int32 recNum, varNt; Byte *tBuffer; int varX;
+    Int32 recNum, varNt; Byte1 *tBuffer; int varX;
 #if LIMITof64K
     long nBytes;
 #endif
@@ -956,6 +961,7 @@ switch (Va->item) {
   case CDF_MAJORITY_:
   case CDF_FORMAT_:
   case CDF_COMPRESSION_:
+  case CDF_LEAPSECONDLASTUPDATED_:
   case ATTR_NAME_:
   case ATTR_SCOPE_:
   case gENTRY_DATA_:
