@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                    CDF `put' operations, part 2.
 *
@@ -169,6 +174,13 @@ switch (Va->item) {
     break;
   }
   /****************************************************************************
+  * CDF_LEAPSECONDLASTUPDATED, 
+  ****************************************************************************/
+  case CDF_LEAPSECONDLASTUPDATED_: {
+    long lastUpdated = va_arg (Va->ap, long);         /* Last updated day. */
+    break;
+  }
+  /****************************************************************************
   * CDF_COMPRESSION_
   ****************************************************************************/
   case CDF_COMPRESSION_: {
@@ -352,7 +364,7 @@ switch (Va->item) {
   case rVAR_COMPRESSION_:
   case zVAR_COMPRESSION_: {
     Logical zOp = (Va->item == zVAR_COMPRESSION_), zVar;
-    Int32 VDRoffset, maxAllocated, pCount; int p;
+    Int32 VDRoffset, maxAllocated, pCount = 0; int p;
     struct CDFstruct *CDF; struct VarStruct *Var;
     struct VDRstruct VDR; struct CPRstruct CPR;
     long cType = va_arg (Va->ap, long);         /* Compression type. */
@@ -384,7 +396,6 @@ switch (Va->item) {
     /**************************************************************************
     * Locate the current variable.
     **************************************************************************/
-
     zVar = CurrentVarMode(CDF,zOp);
     if (zModeON(CDF) || zVar) VDRoffset = CDF->CURzVarOffset;
     else VDRoffset = CDF->CURrVarOffset;
@@ -397,7 +408,6 @@ switch (Va->item) {
     else
         Var = BOO(zOp,CDF->zVars[(int)CDF->CURzVarNum],
                       CDF->rVars[(int)CDF->CURrVarNum]);
-
     /**************************************************************************
     * Read VDR.
     **************************************************************************/
@@ -527,7 +537,7 @@ switch (Va->item) {
     /**************************************************************************
     * Only apply to Version 2.6 and later CDFs.
     **************************************************************************/
-    if ((version*10+release) < 26) return CHECKSUM_NOT_ALLOWED;
+    if ((version*100+release) < 206) return CHECKSUM_NOT_ALLOWED;
 
     /**************************************************************************
     * Get write access.
@@ -563,7 +573,7 @@ switch (Va->item) {
   case rVAR_SPARSERECORDS_:
   case zVAR_SPARSERECORDS_: {
     Logical zOp = (Va->item == zVAR_SPARSERECORDS_), zVar;
-    Int32 VDRoffset, maxAllocated, maxRec;
+    Int32 VDRoffset, maxAllocated, maxRec, flags;
     struct CDFstruct *CDF; struct VarStruct *Var;
     Int32 sRecordsType = (Int32) va_arg (Va->ap, long);
     /**************************************************************************
@@ -608,6 +618,7 @@ switch (Va->item) {
     **************************************************************************/
     if (!sX(ReadVDR(CDF,CDF->fp,VDRoffset,zVar,
 			VDR_MAXREC,&maxRec,
+			VDR_FLAGS,&flags,
 			VDR_NULL),&pStatus)) {
       AbortAccess (CDF, UPDATE, noDELETE);
       return pStatus;
@@ -618,6 +629,8 @@ switch (Va->item) {
       return pStatus;
     }
     if (maxAllocated > NO_RECORD) return CANNOT_SPARSERECORDS;
+    if (!RECvaryBITset(flags) && sRecordsType != NO_SPARSERECORDS)
+      return CANNOT_SPARSERECORDS;
     /**************************************************************************
     * Based on type of sparseness...
     **************************************************************************/
@@ -777,7 +790,8 @@ switch (Va->item) {
 	   CDF->CURgrEntryNum,
 	   CDF->CURgrEntryNum,
 	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
-    if (!ValidDataType(newDataType)) return BAD_DATA_TYPE;
+    if (!ValidDataType(newDataType) || InValidDataType(newDataType))
+      return BAD_DATA_TYPE;
     if (newNumElems != -99 && newNumElems < 1) return BAD_NUM_ELEMS;
     if (!sX(CheckEntryOp(CDF,entryType),&pStatus)) return pStatus;
     eOffset = E3(entryType,CDF->CURgrEntryOffset,
@@ -825,7 +839,8 @@ switch (Va->item) {
 	   CDF->CURgrEntryNum,
 	   CDF->CURgrEntryNum,
 	   CDF->CURzEntryNum) == RESERVED_ENTRYNUM) return NO_ENTRY_SELECTED;
-    if (!ValidDataType((Int32)dataType)) return BAD_DATA_TYPE;
+    if (!ValidDataType((Int32)dataType) || InValidDataType((Int32)dataType))
+      return BAD_DATA_TYPE;
     if (numElems < 1) return BAD_NUM_ELEMS;
     if (!sX(ReadADR(CDF->fp,CDF->CURattrOffset,
 		    ADR_RECORD,&ADR,
