@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF				Get CDF status code explanation text.
 *
@@ -17,6 +22,7 @@
 *   V1.3a  2-Sep-97, J Love	Incremental release in 1997.
 *   V1.4  21-Jun-04, M Liu      Added a new error, NOT_A_CDF_OR_NOT_SUPPORTED.
 *   V1.5  24-Apr-08, M Liu      Modified the copyright.
+*   V1.6  24-Apr-09, M Liu      Modified the copyright.
 *
 ******************************************************************************/
 
@@ -34,11 +40,11 @@ char *copyRight;
   strcpyX (copyRight,
 "\012\
 Common Data Format (CDF)\012\
-(C) Copyright 1990-2008 NASA/GSFC\012\
+(C) Copyright 1990-2017 NASA/GSFC\012\
 Space Physics Data Facility\012\
 NASA/Goddard Space Flight Center\012\
 Greenbelt, Maryland 20771 USA\012\
-(Internet -- CDFSUPPORT@LISTSERV.GSFC.NASA.GOV)\012", CDF_COPYRIGHT_LEN);
+(Internet -- GSFC-CDF-SUPPORT@LISTS.NASA.GOV)\012", CDF_COPYRIGHT_LEN);
   return;
 }
 
@@ -731,13 +737,35 @@ strcpyX (textPtr,
 CDF_STATUSTEXT_LEN);
 break;
 
-/*
-case ZLIB_COMPRESSION_ERROR:
+case IS_A_NETCDF:
 strcpyX (textPtr,
-"ZLIB_COMPRESSION_ERROR: Error during Zlib compression/decompressing.",
+"IS_A_NETCDF: Named CDF file is actually a netCDF file.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case TT2000_TIME_ERROR:
+strcpyX (textPtr,
+"TT2000_TIME_ERROR: Error handling the date/time for TT2000 or a TT2000 epoch.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case UNABLE_TO_PROCESS_CDF:
+strcpyX (textPtr,
+"UNABLE_TO_PROCESS_CDF: One of data fields is invalid... Upgrade the library might fix the problem.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case ZLIB_COMPRESS_ERROR:
+strcpyX (textPtr,
+"ZLIB_ERROR: Error during ZLIB compression.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case ZLIB_UNCOMPRESS_ERROR:
+strcpyX (textPtr,
+"ZLIB_ERROR: Error during ZLIB decompression.",
 CDF_STATUSTEXT_LEN);
 break;
-*/
 
 case ILLEGAL_EPOCH_FIELD:
 strcpyX (textPtr,
@@ -745,14 +773,57 @@ strcpyX (textPtr,
 CDF_STATUSTEXT_LEN);
 break;
 
+case CANNOT_INSERT_RECORDS:
+strcpyX (textPtr,
+"CANNOT_INSERT_RECORDS: Cannot insert records to a variable with sparserecords.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case TT2000_CDF_MAYNEEDUPDATE:
+strcpyX (textPtr,
+"TT2000_CDF_MAYNEEDUPDATE: CDF version or its file-based template may need to upgrade to handle TT2000 data properly (even its value may be valid).",
+CDF_STATUSTEXT_LEN);
+break;
+
+case TT2000_USED_OUTDATED_TABLE:
+strcpyX (textPtr,
+"TT2000_USED_OUTDATED_TABLE: A TT2000 data is either invalid (made with an oudated leap second table) or trying to use an outdated leap second table.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case BADDATE_LEAPSECOND_UPDATED:
+strcpyX (textPtr,
+"BADDATE_LEAPSECOND_UPDATED: The last leap second updated date is not valid (not in the leap second table).",
+CDF_STATUSTEXT_LEN);
+break;
+
+case FUNCTION_NOT_SUPPORTED:
+strcpyX (textPtr,
+"FUNCTION_NOT_SUPPORTED: This function is not supported. Likely, it is trying to access an rVariable.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case BLOCKINGFACTOR_TOO_SMALL:
+strcpyX (textPtr,
+"BLOCKINGFACTOR_TOO_SMALL: The blocking factor set for the compressed variable is smaller than the default. It is changed but may make data access ineffective.",
+CDF_STATUSTEXT_LEN);
+break;
+
+case BLOCKINGFACTOR_TOO_SMALL2:
+strcpyX (textPtr,
+"BLOCKINGFACTOR_TOO_SMALL2: The variable's blocking factor is smaller than the default if it is compressed. The compression is set but may make data access ineffective.",
+CDF_STATUSTEXT_LEN);
+break;
+
 default:
-sprintf (textPtr, "Text not found - unknown CDF status code (%ld).", status);
+snprintf (textPtr, (size_t) CDF_STATUSTEXT_LEN+1,
+          "Text not found - unknown CDF status code (%ld).", status);
 return BAD_CDFSTATUS;
 
   }
 #else
-  sprintf (textPtr,
-	       "Explanation text not available (%ld).", (long) status);
+  snprintf (textPtr, (size_t) CDF_STATUSTEXT_LEN+1,
+	    "Explanation text not available (%ld).", (long) status);
 #endif
   return CDF_OK;
 }
