@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                  Quick Start Test Program (INTERNAL interface/C).
 *
@@ -69,8 +74,8 @@ int main () {
 CDFid id;
 CDFstatus status;
 int dim_n;
-static long encoding = HOST_ENCODING;
-static long actual_encoding = NETWORK_ENCODING;
+static long encoding = IBMPC_ENCODING;
+static long actual_encoding = IBMPC_ENCODING;
 static long majority = ROW_MAJOR;
 static long checkSum = MD5_CHECKSUM, checkSum_out;
 static long numDims = N_DIMS;
@@ -222,6 +227,8 @@ static char zVarsRecBuffer[zDIM_0_SIZEa][zNUM_ELEMSa] = {
   {'%','%','%','%','%','%','%','%'}
 };
 static char zVarsRecBufferOut[zDIM_0_SIZEa][zNUM_ELEMSa];
+long lastUpdated;
+static long lastUpdatedO = 20150701;
 
 /******************************************************************************
 * Display title.
@@ -237,8 +244,8 @@ printf ("Testing Internal/C interface...\n");
 ******************************************************************************/
 
 status = CDFlib (CREATE_, CDF_, "TEST", numDims, dimSizes, &id,
-		 PUT_, CDF_ENCODING_, encoding,
-		       CDF_MAJORITY_, majority,
+/*		 PUT_, CDF_ENCODING_, encoding, */
+		 PUT_, CDF_MAJORITY_, majority,
 		 NULL_);
 
 if (status < CDF_OK) {
@@ -252,8 +259,8 @@ if (status < CDF_OK) {
     if (status < CDF_OK) QuitCDF ("1.1", status);
 
     status = CDFlib (CREATE_, CDF_, "TEST", numDims, dimSizes, &id,
-		     PUT_, CDF_ENCODING_, encoding,
-			   CDF_MAJORITY_, majority,
+/*		     PUT_, CDF_ENCODING_, encoding, */
+		     PUT_, CDF_MAJORITY_, majority,
 		     NULL_);
     if (status < CDF_OK) QuitCDF ("1.2", status);
   }
@@ -308,6 +315,15 @@ status = CDFlib (SELECT_, rVAR_, 0L,
 		 NULL_);
 if (status < CDF_OK) QuitCDF ("2b.04", status);
 
+/******************************************************************************
+* Check leap second last upadted.
+******************************************************************************/
+
+status = CDFlib (GET_, CDF_LEAPSECONDLASTUPDATED_, &lastUpdated,
+                 NULL_);
+if (status < CDF_OK) QuitCDF ("2.7", status);
+/* if ((int)lastUpdated != lastUpdatedO) QuitCDF("2.7a", status); */
+
 /******************************************************************************
 * Close CDF.
 ******************************************************************************/
@@ -338,8 +354,8 @@ if (status < CDF_OK) QuitCDF ("5.0", status);
 ******************************************************************************/
 
 status = CDFlib (CREATE_, CDF_, "TEST", numDims, dimSizes, &id,
-		 PUT_, CDF_ENCODING_, encoding,
-		       CDF_MAJORITY_, majority,
+/*		 PUT_, CDF_ENCODING_, encoding, */
+		 PUT_, CDF_MAJORITY_, majority,
 		 SELECT_, CDF_DECODING_, HOST_DECODING,
 		 NULL_);
 if (status < CDF_OK) QuitCDF ("6.0", status);
