@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF						      SkeletonTable.
 *
@@ -23,6 +28,9 @@
 *   V1.4b 18-Nov-97, J Love	Windows NT/Visual C++.
 *   V1.5  13-Oct-06, M Liu      Changed to allow upper and lower case CDF  
 *                               name to be used on win32.
+*   V3.3  10-Jan-09, M Liu      Validate a file before it is used.
+*   V3.3a 20-Jul-09, M Liu      Added validation for input parameters.
+*   V3.3b 04-Apr-11, M Liu      Modified to handle TT2000 epoch.
 *
 ******************************************************************************/
 
@@ -136,9 +144,14 @@ switch (argC) {
     /**************************************************************************
     * Check if a skeleton table pathname was specified.
     **************************************************************************/
-    if (qop->qualEntered[SKTqual])
-      strcpyX (SKTpath, qop->qualOpt[SKTqual], DU_MAX_PATH_LEN);
-    else {
+    if (qop->qualEntered[SKTqual]) {
+      if (!ValidateQual(qop->qualOpt[SKTqual], validQuals))
+        strcpyX (SKTpath, qop->qualOpt[SKTqual], DU_MAX_PATH_LEN);
+      else {
+        DisplayError ("Invalid \"skeleton\" qualifier.");
+        qopError = TRUE;
+      }
+    } else {
       if (EndsWithIgCase(CDFname, ".cdf"))
         RemoveCDFFileExtension (CDFname, SKTpath);
       else
@@ -171,19 +184,24 @@ switch (argC) {
 	  break;
 	}
 	if (qop->qualEntered[VALUESqual]) {
-	  size_t len = strlen(qop->qualOpt[VALUESqual]);
-	  if (strcmpIgCase(qop->qualOpt[VALUESqual],"none") != 0)
-	    varValues = NOvalues;
-	  else if (strcmpIgCase(qop->qualOpt[VALUESqual],"nrv") != 0)
-	    varValues = NRVvalues;
-	  else if (strcmpIgCase(qop->qualOpt[VALUESqual],"rv") != 0)
-	    varValues = RVvalues;
-	  else if (strcmpIgCase(qop->qualOpt[VALUESqual],"all") != 0)
-	    varValues = ALLvalues;
-	  else {
-	    varValues = NAMEDvalues;
- 	    variables = cdf_AllocateMemory (len + 1, FatalError);
- 	    strcpyX (variables, qop->qualOpt[VALUESqual], len);
+          if (ValidateQual(qop->qualOpt[VALUESqual], validQuals)) {
+            DisplayError ("Invalid \"values\" qualifier.");
+            qopError = TRUE;
+          } else {
+	    size_t len = strlen(qop->qualOpt[VALUESqual]);
+	    if (strcmpIgCase(qop->qualOpt[VALUESqual],"none") == 1)
+	      varValues = NOvalues;
+	    else if (strcmpIgCase(qop->qualOpt[VALUESqual],"nrv") == 1)
+	      varValues = NRVvalues;
+	    else if (strcmpIgCase(qop->qualOpt[VALUESqual],"rv") == 1)
+	      varValues = RVvalues;
+	    else if (strcmpIgCase(qop->qualOpt[VALUESqual],"all") == 1)
+	      varValues = ALLvalues;
+	    else {
+	      varValues = NAMEDvalues;
+ 	      variables = cdf_AllocateMemory ((size_t)len + 1, FatalError);
+ 	      strcpyX (variables, qop->qualOpt[VALUESqual], len);
+	    }
 	  }
 	}
 	break;
@@ -277,7 +295,8 @@ switch (argC) {
     if (qopError) return FALSE;
     break;
 }
-  
+
+/* CDFsetValidate (VALIDATEFILEon); */
 /******************************************************************************
 * Create skeleton table.
 ******************************************************************************/
@@ -348,7 +367,7 @@ if (!StatusHandlerC2S(status)) return FALSE;
 
 CHECKforABORTso
 if (mLog) WriteOut (stdout, "Printing Header Information...\n");
-if (!WriteHeader(CDFname)) return FALSE;
+if (!WriteHeader(CDFname, varValues, variables)) return FALSE;
 
 /******************************************************************************
 * Write global scope attributes section.
@@ -408,8 +427,10 @@ return TRUE;
 * WriteHeader.
 ******************************************************************************/
 
-Logical WriteHeader (CDFname)
+Logical WriteHeader (CDFname, varValues, variables)
 char *CDFname;
+int varValues;
+char *variables;
 {
 CDFstatus status;
 long encoding, majority, format;
@@ -417,10 +438,11 @@ long numDims;
 long dimSizes[CDF_MAX_DIMS];
 long numVars, numZvars, maxRec;
 long numVattrs, numGattrs;
-int dimN;
-long version, release, increment;
+int  dimN;
+long lastUpdated = -1L, version, release, increment;
 char sizes[MAX_SIZES_LEN+1], subIncrement;
 char text[MAX_TEXT_LEN+1], tempS[MAX_LINE_LEN+1];
+long compress, compParms[CDF_MAX_DIMS], compPct, checkSum;
 
 status = CDFlib (GET_, CDF_VERSION_, &version,
 		       CDF_RELEASE_, &release,
@@ -428,6 +450,12 @@ status = CDFlib (GET_, CDF_VERSION_, &version,
 		 NULL_);
 if (!StatusHandlerC2S(status)) return FALSE;
 
+if (version > 2) {
+  status = CDFlib (GET_, CDF_LEAPSECONDLASTUPDATED_, &lastUpdated,
+ 		   NULL_);
+  if (!StatusHandlerC2S(status)) return FALSE;
+} 
+
 WriteOutFP (SKTfp, "\n! CDF created/modified by CDF V%ld.%ld.%ld",
 	    version, release, increment);
 
@@ -438,8 +466,12 @@ status = CDFlib (GET_, LIB_VERSION_, &version,
 		 NULL_);
 if (!StatusHandlerC2S(status)) return FALSE;
 
-WriteOutFP (SKTfp, "\n! Skeleton table created by CDF V%ld.%ld.%ld%c",
-	    version, release, increment, subIncrement);
+if (subIncrement != ' ')
+  WriteOutFP (SKTfp, "\n! Skeleton table created by CDF V%ld.%ld.%ld_%c",
+	      version, release, increment, subIncrement);
+else
+  WriteOutFP (SKTfp, "\n! Skeleton table created by CDF V%ld.%ld.%ld",
+              version, release, increment);
 
 WriteOut (SKTfp, "\n");
 WriteOut (SKTfp, "\n#header");
@@ -473,7 +505,8 @@ WriteOut (SKTfp, FormatToken(format));
 if (numDims > 0) {
   sizes[0] = NUL;
   for (dimN = 0; dimN < numDims; dimN++) {
-     sprintf (&sizes[strlen(sizes)], "%ld ", dimSizes[dimN]);
+     snprintf (&sizes[strlen(sizes)], (size_t) sizeof(sizes)-strlen(sizes),
+	       "%ld ", dimSizes[dimN]);
   }
   sizes[strlen(sizes)-1] = NUL;			/* Wipe out trailing blank. */
 }
@@ -494,10 +527,11 @@ Ncharacters (SKTfp, MaxInt((int)strlen("Sizes"),(int)strlen(sizes)), (int)'-');
 
 WriteOut (SKTfp, "\n");
 Ncharacters (SKTfp, 2, (int) ' ');
-sprintf (text, "%ld/%ld", numVars, numZvars);
+snprintf (text, (size_t) sizeof(text), "%ld/%ld", numVars, numZvars);
 Ncharacters (SKTfp, MaxInt((int)(6 - strlen(text)),0), (int) ' ');
-sprintf (tempS, "%s     %7ld       %7ld     %5ld/z    %3ld",
-	 text, numGattrs, numVattrs, maxRec + 1, numDims);
+snprintf (tempS, (size_t) sizeof(tempS),
+	  "%s     %7ld       %7ld     %5ld/z    %3ld",
+	  text, numGattrs, numVattrs, maxRec + 1, numDims);
 WriteOut (SKTfp, tempS);
 
 if (numDims > 0) {
@@ -506,6 +540,44 @@ if (numDims > 0) {
   WriteOut (SKTfp, sizes);
 }
 
+status = CDFlib (GET_, CDF_COMPRESSION_, &compress, compParms, &compPct,
+                 NULL_);
+if (!StatusHandlerC2S(status)) return FALSE;
+
+/*
+if (compress != NO_COMPRESSION || compress != NO_CHECKSUM)
+  WriteOutFP (SKTfp, "\n");
+*/
+
+/* if (compress != NO_COMPRESSION) */
+  WriteOutFP (SKTfp, "\n! CDF_COMPRESSION: %s",
+              CompressionToken(compress, compParms));
+  WriteOutFP (SKTfp, "\n! (Valid compression: None, GZIP.1-9, RLE.0, HUFF.0, AHUFF.0)");
+
+status = CDFlib (GET_, CDF_CHECKSUM_, &checkSum,
+                 NULL_);
+if (!StatusHandlerC2S(status)) return FALSE;
+
+/* if (checkSum != NO_CHECKSUM) */
+WriteOutFP (SKTfp, "\n! CDF_CHECKSUM: %s", ChecksumToken(checkSum)); 
+WriteOutFP (SKTfp, "\n! (Valid checksum: None, MD5)");
+
+if (!PriorTo ("3.5.0", version, release, increment)) {
+  if (lastUpdated > -1 || WriteTT2000VarData(varValues, variables)) {
+    if (lastUpdated > -1)
+      WriteOutFP (SKTfp, "\n! CDF_LEAPSECONDLASTUPDATED: %ld", lastUpdated); 
+    else {
+      long year, month, day;
+#if defined(vms)
+      CDFgetLastDateinLeapSecondsTBL (&year, &month, &day);
+#else
+      CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
+#endif
+      WriteOutFP (SKTfp, "\n! CDF_LEAPSECONDLASTUPDATED: %ld", 10000*year+
+                                                               100*month+day);
+    }
+  }
+}
 return TRUE;
 }
 
@@ -581,7 +653,8 @@ if (numAttrs > 0) {
 
        delim = PickDelimiter (attrName, strlen(attrName));
        WriteOut (SKTfp, "\n\n");
-       sprintf (tempS, "  %c%s%c", delim, attrName, delim);
+       snprintf (tempS, (size_t) sizeof(tempS),
+	         "  %c%s%c", delim, attrName, delim);
        ccc = WriteOut (SKTfp, tempS);
 
        /***********************************************************************
@@ -600,8 +673,7 @@ if (numAttrs > 0) {
 	    entryCount++;
 
 	    value = cdf_AllocateMemory ((size_t) (CDFelemSize(dataType) *
-					      numElements),
-				    FatalError);
+					numElements), FatalError);
 
 	    status = CDFlib (GET_, gENTRY_DATA_, value,
 			     NULL_);
@@ -621,10 +693,20 @@ if (numAttrs > 0) {
 	    Ncharacters (SKTfp, ENTRY_NUM_COL - ccc, (int) ' ');
 	    ccc += ENTRY_NUM_COL - ccc;
 
-	    sprintf (tempS, "%*ld:    %s   ", ENTRY_NUM_WIDTH, entryN + 1,
-		     DataTypePaddedString(dataType));
+	    snprintf (tempS, (size_t) sizeof(tempS),
+		      "%*ld:    %s   ", ENTRY_NUM_WIDTH, entryN + 1,
+		      DataTypePaddedString(dataType));
 	    ccc += WriteOut (SKTfp, tempS);
 
+	    if (STRINGdataType (dataType)) { /* not allow LF and CR */
+	      int ii;
+	      for (ii = 0; ii < (int) numElements; ++ii) { 
+	        if (((int)*(((char *)value)+ii) == (int) '\n') ||
+	            ((int)*(((char *)value)+ii) == (int) '\r'))
+		  memcpy (((char *)value)+ii, " ", 1);
+	      }
+	    }
+
 	    ccc += WriteOut (SKTfp, "{ ");
 	    WriteEntryValue (SKTfp, dataType, numElements, value, ccc,
 			     MAX_COL_TO_USE - 4);
@@ -639,6 +721,12 @@ if (numAttrs > 0) {
        * Write '.' after last entry.
        ***********************************************************************/
        WriteOut (SKTfp, " .");
+       if (!ISTPname(attrName)) {
+         WriteOut (SKTfp, "\n");
+         WriteOutFP (SKTfp,
+                     "!  Warning: \"%s\": Not ISTP compliant... not recommended",
+                     attrName);
+       }
      }
      CHECKforABORTso
   }
@@ -685,8 +773,15 @@ if (numAttrs > 0) {
        ***********************************************************************/
        VattrCount++;
        delim = PickDelimiter (attrName, strlen(attrName));
-       sprintf (tempS, "\n  %c%s%c", delim, attrName, delim);
+       snprintf (tempS, (size_t) sizeof(tempS),
+		 "\n  %c%s%c", delim, attrName, delim);
        WriteOut (SKTfp, tempS);
+       if (!ISTPname(attrName)) {
+         WriteOut (SKTfp, "\n");
+         WriteOutFP (SKTfp,
+                     "!  Warning: \"%s\": Not ISTP compliant... not recommended",
+                     attrName);
+       }
      }
      CHECKforABORTso
   }
@@ -764,12 +859,18 @@ long attrN;
 long scope;
 int dimN;
 int ccc;			      /* Current Cursor Column (base is 0). */
+int ddd;
 void *value;
 int VattrCount;
 char varys[MAX_VARYS_LEN+1];
 char sizes[MAX_SIZES_LEN+1];
 char tempS[MAX_LINE_LEN+1];
+long cType, cParms[CDF_MAX_DIMS], cPct, sparse, bf;
+void *pad = NULL;
+int padValue;
+long sarray[CDF_MAX_PARMS];
 
+sarray[0] = 0;
 status = CDFlib (SELECT_, VAR(zVar), varN,
 		 GET_, VAR_NAME(zVar), varName,
 		       VAR_DATATYPE(zVar), &varDataType,
@@ -777,6 +878,9 @@ status = CDFlib (SELECT_, VAR(zVar), varN,
 		       VAR_RECVARY(zVar), &recVary,
 		       VAR_DIMVARYS(zVar), dimVarys,
 		       VAR_MAXREC(zVar), &maxRec,
+		       VAR_COMPRESSION(zVar), &cType, cParms, &cPct,
+		       VAR_SPARSERECORDS(zVar), &sparse,
+		       VAR_BLOCKINGFACTOR(zVar), &bf,
 		 NULL_);
 if (!StatusHandlerC2S(status)) return FALSE;
 
@@ -791,6 +895,18 @@ else
 		   NULL_);
 if (!StatusHandlerC2S(status)) return FALSE;
 
+pad = cdf_AllocateMemory ((size_t) ((varNumElements + 
+                                     (STRINGdataType(varDataType)? 1 : 0)) *
+                                    CDFelemSize(varDataType)),
+                          FatalError);
+status = CDFlib (GET_, VAR_PADVALUE(zVar), pad,
+                 NULL_);
+if (!StatusHandlerC2S(status)) return FALSE;
+if (status != NO_PADVALUE_SPECIFIED)
+  padValue = 1;
+else
+  padValue = 0;
+
 /***************************************************************************
 * If zVariable, build dimension sizes string.
 ***************************************************************************/
@@ -798,7 +914,8 @@ if (!StatusHandlerC2S(status)) return FALSE;
 if (numDims > 0) {
   sizes[0] = NUL;
   for (dimN = 0; dimN < numDims; dimN++) {
-     sprintf (&sizes[strlen(sizes)], "%ld ", dimSizes[dimN]);
+     snprintf (&sizes[strlen(sizes)], (size_t) sizeof(sizes)-strlen(sizes),
+	       "%ld ", dimSizes[dimN]);
   }
   sizes[strlen(sizes)-1] = NUL;			/* Wipe out trailing blank. */
 }
@@ -888,24 +1005,29 @@ WriteOut (SKTfp, "\n");
 
 delim = PickDelimiter (varName, strlen(varName));
 WriteOut (SKTfp, "\n");
-sprintf (tempS, "  %c%s%c", delim, varName, delim);
+snprintf (tempS, (size_t) sizeof(tempS), "  %c%s%c", delim, varName, delim);
 ccc = WriteOut (SKTfp, tempS);
 
-if (ccc > VAR_DATATYPE_COL - 3) {
+if (varDataType != CDF_TIME_TT2000) ddd = 0;
+else ddd = 4;
+
+if (ccc > (VAR_DATATYPE_COL - 3 - ddd)) {
   WriteOut (SKTfp, "\n");
   ccc = 0;
 }
 
-Ncharacters (SKTfp, VAR_DATATYPE_COL - ccc, (int) ' ');
+Ncharacters (SKTfp, VAR_DATATYPE_COL - ccc - ddd, (int) ' ');
 
-sprintf (tempS, "%s %*ld   ", DataTypePaddedString(varDataType),
-	 VAR_NUMELEMS_WIDTH, varNumElements);
+snprintf (tempS, (size_t) sizeof(tempS),
+	  "%s %*ld   ", DataTypePaddedString(varDataType),
+	  VAR_NUMELEMS_WIDTH, varNumElements);
 WriteOut (SKTfp, tempS);
 
 Ncharacters (SKTfp, 2, (int) ' ');
 
 if (zVar) {
-  sprintf (tempS, " %*ld ", zVAR_NUMDIMS_WIDTH, numDims);
+  snprintf (tempS, (size_t) sizeof(tempS),
+	    " %*ld ", zVAR_NUMDIMS_WIDTH, numDims);
   WriteOut (SKTfp, tempS);
   Ncharacters (SKTfp, 2, (int) ' ');
   Ncharacters (SKTfp, FRONTpadVALUE("Sizes",sizes), (int) ' ');
@@ -924,6 +1046,72 @@ if (numDims > 0) {
   WriteOut (SKTfp, varys);
 }
 
+WriteOutFP (SKTfp, "\n");
+
+if (!ISTPname(varName)) {
+  WriteOutFP (SKTfp, "!  Warning: \"%s\": Not ISTP compliant... not recommended",
+              varName);
+}
+
+/* if (cType != NO_COMPRESSION) */
+  WriteOutFP (SKTfp, "\n  ! VAR_COMPRESSION: %s",
+              CompressionToken(cType, cParms));
+  WriteOutFP (SKTfp, "\n  ! (Valid compression: None, GZIP.1-9, RLE.0, HUFF.0, AHUFF.0)");
+
+/* if (recVary == VARY) { */
+  WriteOutFP (SKTfp, "\n  ! VAR_SPARSERECORDS: %s",
+            SparsenessToken(sparse, NO_SPARSEARRAYS, sarray));
+  WriteOutFP (SKTfp, "\n  ! (Valid sparserecords: None, sRecords.PAD, sRecords.PREV)");
+/* } */
+
+if (bf != 0) {
+  WriteOutFP (SKTfp, "\n  ! VAR_BLOCKINGFACTOR: %ld", bf);
+}
+
+if (STRINGdataType(varDataType)) {
+  int ix, iy;
+  if (padValue == 0) { /* Only output a single blank */
+    *(((char *) pad)+0) = (char) DEFAULT_CHAR_PADVALUE;
+    *(((char *) pad)+1) = (char) '\0';
+  } else {
+    long numBytes = (long) UTF8StrLength ((unsigned char *)pad);
+    iy = -1;
+    if (numBytes == varNumElements) {
+      for (ix = 0; ix < (int) varNumElements; ++ix)
+        if (!Printable(*(((char *) pad)+ix))) *(((char *) pad)+ix) = '.';
+    }
+    for (ix = (varNumElements-1); ix >= 0; --ix) {
+      if (*(((char *) pad)+ix) != ' ') {
+        iy = ix;
+        break;
+      }
+    }
+    if (iy == -1) /* All blanks... output a single blank */
+      *(((char *) pad)+1) = (char) '\0';
+    else
+      *(((char *) pad)+iy+1) = (char) '\0';
+  }
+  if (strchr(pad, '"') == NULL) 
+    WriteOutFP (SKTfp, "\n  ! VAR_PADVALUE: \"%s\"", pad);
+  else if (strchr(pad, '\'') == NULL)
+    WriteOutFP (SKTfp, "\n  ! VAR_PADVALUE: '%s'", pad);
+  else
+    WriteOutFP (SKTfp, "\n  ! VAR_PADVALUE: $%s$", pad);
+} else {
+  char string[101], *format;
+  int  style;
+  if (TT2000dataType(varDataType)) style = TT2000_3_STYLE;
+  else style = EPOCH0_STYLE;
+  status = GetFormatEntry (zVar, varN, &format);
+  if (padValue == 1 && useFormat && format != NULL)
+    EncodeValue (varDataType, pad, string, style, 100);
+  else
+    EncodeValue (varDataType, pad, string, style, 100);
+    WriteOutFP (SKTfp, "\n  ! VAR_PADVALUE: %s", string);
+  if (format != NULL) cdf_FreeMemory (format, FatalError);
+}
+cdf_FreeMemory (pad, FatalError);
+
 CHECKforABORTso
 
 /***************************************************************************
@@ -988,20 +1176,33 @@ for (attrN = 0; attrN < numAttrs; attrN++) {
 
        delim = PickDelimiter (attrName, strlen(attrName));
        WriteOut (SKTfp, "\n");
-       sprintf (tempS, "    %c%s%c", delim, attrName, delim);
+       snprintf (tempS, (size_t) sizeof(tempS),
+		 "    %c%s%c", delim, attrName, delim);
        ccc = WriteOut (SKTfp, tempS);
 
-       if (ccc > ENTRY_DATATYPE_COL - 3) {
+       if (entryDataType != CDF_TIME_TT2000) ddd = 0;
+       else ddd = 4;
+
+       if (ccc > (ENTRY_DATATYPE_COL - 2 - ddd)) {
 	 WriteOut (SKTfp, "\n");
 	 ccc = 0;
        }
 
-       Ncharacters (SKTfp, ENTRY_DATATYPE_COL - ccc, (int) ' ');
+       Ncharacters (SKTfp, ENTRY_DATATYPE_COL - ccc - ddd, (int) ' ');
        ccc += ENTRY_DATATYPE_COL - ccc;
 
        ccc += WriteOut (SKTfp, DataTypePaddedString(entryDataType));
        ccc += WriteOut (SKTfp, "   ");
 
+       if (STRINGdataType (entryDataType)) { /* not allow LF and CR */
+         int ii;
+         for (ii = 0; ii < (int) entryNumElements; ++ii) {
+           if (((int)*(((char *)value)+ii) == (int) '\n') ||
+               ((int)*(((char *)value)+ii) == (int) '\r'))
+             memcpy (((char *)value)+ii, " ", 1);
+         }
+       }
+
        ccc += WriteOut (SKTfp, "{ ");
        WriteEntryValue (SKTfp, entryDataType, entryNumElements,
 			value, ccc, MAX_COL_TO_USE - 4);
@@ -1131,6 +1332,9 @@ long maxRec;
   long nRecValues, i, recN;
   int dimN;
   char Evalue[80+1], *format, tempS[MAX_LINE_LEN+1];
+  int style;
+  if (TT2000dataType(dataType)) style = TT2000_3_STYLE;
+  else style = EPOCH0_STYLE;
   status = CDFlib (SELECT_, VAR(zVar), varN,
 		   GET_, CDF_MAJORITY_, &majority,
 		   NULL_);
@@ -1148,7 +1352,7 @@ long maxRec;
     }
   }
   value = cdf_AllocateMemory ((size_t) (numElements * CDFelemSize(dataType)),
-			  FatalError);
+			      FatalError);
   if (useFormat) {
     status = GetFormatEntry (zVar, varN, &format);
     if (!StatusHandlerC2S(status)) {
@@ -1167,6 +1371,7 @@ long maxRec;
        if (format != NULL) cdf_FreeMemory (format, FatalError);
        return FALSE;
      }
+     if (status == VIRTUAL_RECORD_DATA) continue;
      for (i = 0; i < nRecValues; i++) {
         status = CDFlib (SELECT_, BOO(zVar,zVAR_DIMINDICES_,
 				           rVARs_DIMINDICES_), indices,
@@ -1177,15 +1382,17 @@ long maxRec;
 	  if (format != NULL) cdf_FreeMemory (format, FatalError);
 	  return FALSE;
 	}
+        if (status == VIRTUAL_RECORD_DATA) break;
         ccc = WriteOut (fp, "\n    ");
 	if (recVary) {
-	  sprintf (tempS, "%ld:[", recN + 1);	  
+	  snprintf (tempS, (size_t) sizeof(tempS), "%ld:[", recN + 1);	  
 	}
 	else
 	  strcpyX (tempS, "[", MAX_LINE_LEN);
         if (numDims > 0) {
           for (dimN = 0; dimN < numDims; dimN++) {
-             sprintf (EofS(tempS), "%ld", indices[dimN] + 1);
+             snprintf (EofS(tempS), (size_t) sizeof(tempS)-strlen(tempS),
+		       "%ld", indices[dimN] + 1);
 	     if (dimN != numDims-1) strcatX (tempS, ",", MAX_LINE_LEN);
           }
         }
@@ -1198,8 +1405,29 @@ long maxRec;
           WriteOut (fp, " }");
         }
         else {
-          EncodeValuesFormat (dataType, 1L, value, Evalue,
-			      format, 0, 0, EPOCH0_STYLE);
+          if (useFormat && format != NULL) {
+            if ((dataType == CDF_FLOAT) || (dataType == CDF_REAL4)) {
+              if (!isnan((double)*(float *)value) &&
+                  *(float *)value <= DEFAULT_FLOAT_PADVALUE) {
+                EncodeValueFormat (dataType, value, Evalue, NULL, 0, 80,
+	 	                   style, (size_t) sizeof(Evalue));
+              } else 
+                EncodeValue (dataType, value, Evalue, style,
+                             (size_t) sizeof(Evalue));
+            } else if ((dataType == CDF_DOUBLE) || (dataType == CDF_REAL8)) {
+              if (!isnan(*(double *)value) &&
+                  *(double *)value <= DEFAULT_DOUBLE_PADVALUE) {
+                EncodeValueFormat (dataType, value, Evalue, NULL, 0, 80,
+	 	                   style, (size_t) sizeof(Evalue));
+              } else
+                EncodeValue (dataType, value, Evalue, style,
+                             (size_t) sizeof(Evalue));
+            } else
+              EncodeValueFormat (dataType, value, Evalue, format, 0, 80,
+	 	               style, (size_t) sizeof(Evalue));
+          } else
+            EncodeValue (dataType, value, Evalue, style,
+                         (size_t) sizeof(Evalue));
 						/* Don't care about width. */
           WriteOut (fp, Evalue);
         }
@@ -1225,6 +1453,7 @@ Logical VariableSelected (varName, variables)
 char *varName;
 char *variables;
 {
+/*
   char *d1, *d2, varNameT[CDF_VAR_NAME_LEN256+1];
   size_t len;
   d1 = variables;
@@ -1242,6 +1471,12 @@ char *variables;
     if (*d1 == NUL) return FALSE;
     d1++;
   }
+*/
+  if (variables == NULL) return FALSE;
+  if (strstr((const char *)variables, (const char *)varName) != NULL)
+    return TRUE;
+  else
+    return FALSE;
 }
 
 /******************************************************************************
@@ -1257,6 +1492,7 @@ switch (dataType) {
   case CDF_INT1:   return "CDF_INT1  ";
   case CDF_INT2:   return "CDF_INT2  ";
   case CDF_INT4:   return "CDF_INT4  ";
+  case CDF_INT8:   return "CDF_INT8  ";
   case CDF_UINT1:  return "CDF_UINT1 ";
   case CDF_UINT2:  return "CDF_UINT2 ";
   case CDF_UINT4:  return "CDF_UINT4 ";
@@ -1268,6 +1504,7 @@ switch (dataType) {
   case CDF_CHAR:   return "CDF_CHAR  ";
   case CDF_UCHAR:  return "CDF_UCHAR ";
   case CDF_EPOCH16:  return "CDF_EPOCH16";
+  case CDF_TIME_TT2000:  return "CDF_TIME_TT2000";
 }
 return "??????????";
 }
@@ -1325,6 +1562,85 @@ CDFstatus status;
   return TRUE;		/* CDF_OK */
 }
 
+/******************************************************************************
+* WriteTT2000VarData.
+******************************************************************************/
+
+Logical WriteTT2000VarData (varValues, variables)
+int varValues;
+char *variables;
+{
+CDFstatus status;
+long numVars, numZvars;
+long varN, recVary, varDataType, maxRec;
+char varName[CDF_VAR_NAME_LEN256+1];
+
+if (varValues == NOvalues) return FALSE;
+status = CDFlib (GET_, CDF_NUMrVARS_, &numVars,
+		       CDF_NUMzVARS_, &numZvars,
+		 NULL_);
+if (!StatusHandlerC2S(status)) return FALSE;
+if (numVars > 0) {
+  for (varN = 0; varN < numVars; varN++) {
+    status = CDFlib (SELECT_, rVAR_, varN,
+                     GET_, rVAR_NAME_, varName,
+                           rVAR_DATATYPE_, &varDataType,
+                           rVAR_RECVARY_, &recVary,
+                           rVAR_MAXREC_, &maxRec,
+                     NULL_);
+    if (!StatusHandlerC2S(status)) return FALSE;
+    if (varDataType == CDF_TIME_TT2000) {
+      switch (varValues) {
+        case NRVvalues:
+          if (!recVary && maxRec == 0) return TRUE;
+          break;
+        case RVvalues:
+          if (!recVary && maxRec > -1) return TRUE;
+          break;
+        case ALLvalues:
+          if (maxRec > -1) return TRUE;
+          break;
+        case NAMEDvalues:
+          if (VariableSelected(varName,variables) && maxRec > -1) return TRUE;
+          break;
+        default:
+          break; 
+      }
+    }
+  }
+}
+if (numZvars > 0) {
+  for (varN = 0; varN < numZvars; varN++) {
+    status = CDFlib (SELECT_, zVAR_, varN,
+                     GET_, zVAR_NAME_, varName,
+                           zVAR_DATATYPE_, &varDataType,
+                           zVAR_RECVARY_, &recVary,
+                           zVAR_MAXREC_, &maxRec,
+                     NULL_);
+    if (!StatusHandlerC2S(status)) return FALSE;
+    if (varDataType == CDF_TIME_TT2000) {
+      switch (varValues) {
+        case NRVvalues:
+          if (!recVary && maxRec == 0) return TRUE;
+          break;
+        case RVvalues:
+          if (!recVary && maxRec > -1) return TRUE;
+          break;
+        case ALLvalues:
+          if (maxRec > -1) return TRUE;
+          break;
+        case NAMEDvalues:
+          if (VariableSelected(varName,variables) && maxRec > -1) return TRUE;
+          break;
+        default:
+          break; 
+      }
+    }
+  }
+}
+return FALSE;
+}
+
 /******************************************************************************
 * SkeletonTableQOPs.
 *    Returns TRUE if execution should continue.
@@ -1424,85 +1740,85 @@ char **argV[];
       * Ok.
       ************************************************************************/
       case OKin: {
-		char *valuesOptions[5] = { "off", "nrv", "rv", "all", NULL };
-		int n;
-		char tempS1[1+1];
-		/**********************************************************************
-		* Get the value of each control.
-		**********************************************************************/
-		GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		GetIText ((Handle) controlHs[SKTTEXTin], sktText);
-		GetIText ((Handle) controlHs[CACHEin], cacheText);
-		GetIText ((Handle) controlHs[VARSin], varsText);
-		outToScreen = GetCtlValue (controlHs[SCREENin]);
-		useFormat = GetCtlValue (controlHs[FORMATin]);
-		negToPos = GetCtlValue (controlHs[NEGZin]);
-		pageOutput = GetCtlValue (controlHs[PAGEin]);
-		reportInfos = GetCtlValue (controlHs[INFOin]);
-		reportWarns = GetCtlValue (controlHs[WARNin]);
-		reportErrors = GetCtlValue (controlHs[ERRORin]);
-		dispStats = GetCtlValue (controlHs[STATSin]);
-		logMsg = GetCtlValue (controlHs[LOGin]);
-		for (varValues = 0; varValues < 5; varValues++) {
-		   if (GetCtlValue(controlHs[VALUESinBASE+varValues])) break;
-		}
-		for (zModeSrc = 0; zModeSrc < 3; zModeSrc++) {
-		   if (GetCtlValue(controlHs[ZMODEinBASE+zModeSrc])) break;
-		}
-		/**********************************************************************
-		* Build argc/argv.
-		**********************************************************************/
-		*argC = 13 + BOO(NULpString(CDFtext),0,1) +
-			     	 BOO(NULpString(sktText),0,2) +
-			     	 BOO(NULpString(cacheText),0,2);
-    	*argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *),
+	char *valuesOptions[5] = { "off", "nrv", "rv", "all", NULL };
+	int n;
+	char tempS1[1+1];
+	/**********************************************************************
+	* Get the value of each control.
+	**********************************************************************/
+	GetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	GetIText ((Handle) controlHs[SKTTEXTin], sktText);
+	GetIText ((Handle) controlHs[CACHEin], cacheText);
+	GetIText ((Handle) controlHs[VARSin], varsText);
+	outToScreen = GetCtlValue (controlHs[SCREENin]);
+	useFormat = GetCtlValue (controlHs[FORMATin]);
+	negToPos = GetCtlValue (controlHs[NEGZin]);
+	pageOutput = GetCtlValue (controlHs[PAGEin]);
+	reportInfos = GetCtlValue (controlHs[INFOin]);
+	reportWarns = GetCtlValue (controlHs[WARNin]);
+	reportErrors = GetCtlValue (controlHs[ERRORin]);
+	dispStats = GetCtlValue (controlHs[STATSin]);
+	logMsg = GetCtlValue (controlHs[LOGin]);
+	for (varValues = 0; varValues < 5; varValues++) {
+	   if (GetCtlValue(controlHs[VALUESinBASE+varValues])) break;
+	}
+	for (zModeSrc = 0; zModeSrc < 3; zModeSrc++) {
+	   if (GetCtlValue(controlHs[ZMODEinBASE+zModeSrc])) break;
+	}
+	/**********************************************************************
+	* Build argc/argv.
+	**********************************************************************/
+	*argC = 13 + BOO(NULpString(CDFtext),0,1) +
+		     	 BOO(NULpString(sktText),0,2) +
+		     	 BOO(NULpString(cacheText),0,2);
+    	*argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *),
 						  FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
-		MAKEbooARGv (argV, n, outToScreen, "-screen", "-noscreen")
-		MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
-		MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
-		MAKEbooARGv (argV, n, pageOutput, "-page", "-nopage")
-		MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
-		MAKEbooARGv (argV, n, logMsg, "-log", "-nolog")
-		MAKEstrARGv (argV, n, "-zmode")
-		sprintf (tempS1, "%d", zModeSrc);
-		MAKEstrARGv (argV, n, tempS1)
-		MAKEstrARGv (argV, n, "-report")
-		MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
+	MAKEbooARGv (argV, n, outToScreen, "-screen", "-noscreen")
+	MAKEbooARGv (argV, n, useFormat, "-format", "-noformat")
+	MAKEbooARGv (argV, n, negToPos, "-neg2posfp0", "-noneg2posfp0")
+	MAKEbooARGv (argV, n, pageOutput, "-page", "-nopage")
+	MAKEbooARGv (argV, n, dispStats, "-statistics", "-nostatistics")
+	MAKEbooARGv (argV, n, logMsg, "-log", "-nolog")
+	MAKEstrARGv (argV, n, "-zmode")
+	snprintf (tempS1, (size_t) sizeof(tempS1), "%d", zModeSrc);
+	MAKEstrARGv (argV, n, tempS1)
+	MAKEstrARGv (argV, n, "-report")
+	MAKEstrARGv (argV, n, StatusCodeReportOptions(reportErrors,
 						      reportWarns,
 						      reportInfos))
-		MAKEstrARGv (argV, n, "-values")
-		switch (varValues) {
-		  case NAMEDvalues:
-		    PtoCstr (varsText);
-		    MAKEstrARGv (argV, n, (char *) varsText)
-		    CtoPstr ((char *) varsText);
-		    break;
-		  default:
-		    MAKEstrARGv (argV, n, valuesOptions[varValues])
-		    break;
-		}
-		if (!NULpString(sktText)) {
-		  MAKEstrARGv (argV, n, "-skeleton")
-		  PtoCstr (sktText);
-		  MAKEstrARGv (argV, n, (char *) sktText)
-		  CtoPstr ((char *) sktText);
-		}
-		if (!NULpString(cacheText)) {
-		  MAKEstrARGv (argV, n, "-cache")
-		  PtoCstr (cacheText);
-		  MAKEstrARGv (argV, n, (char *) cacheText)
-		  CtoPstr ((char *) cacheText);
-		}
-		if (!NULpString(CDFtext)) {
-		  PtoCstr (CDFtext);
-		  MAKEstrARGv (argV, n, (char *) CDFtext)
-		  CtoPstr ((char *) CDFtext);
-		}
-		/**********************************************************************
-		* Close the dialog and return.
-		**********************************************************************/
+	MAKEstrARGv (argV, n, "-values")
+	switch (varValues) {
+	  case NAMEDvalues:
+	    PtoCstr (varsText);
+	    MAKEstrARGv (argV, n, (char *) varsText)
+	    CtoPstr ((char *) varsText);
+	    break;
+	  default:
+	    MAKEstrARGv (argV, n, valuesOptions[varValues])
+	    break;
+	}
+	if (!NULpString(sktText)) {
+	  MAKEstrARGv (argV, n, "-skeleton")
+	  PtoCstr (sktText);
+	  MAKEstrARGv (argV, n, (char *) sktText)
+	  CtoPstr ((char *) sktText);
+	}
+	if (!NULpString(cacheText)) {
+	  MAKEstrARGv (argV, n, "-cache")
+	  PtoCstr (cacheText);
+	  MAKEstrARGv (argV, n, (char *) cacheText)
+	  CtoPstr ((char *) cacheText);
+	}
+	if (!NULpString(CDFtext)) {
+	  PtoCstr (CDFtext);
+	  MAKEstrARGv (argV, n, (char *) CDFtext)
+	  CtoPstr ((char *) CDFtext);
+	}
+	/**********************************************************************
+	* Close the dialog and return.
+	**********************************************************************/
         CloseDialog (dialogP);
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
@@ -1516,14 +1832,14 @@ char **argV[];
       case HELPin: {
         int n;
         *argC = 1;
-        *argV = (char **) cdf_AllocateMemory (*argC * sizeof(char *), FatalError);
-		n = 0;
-		MAKEstrARGv (argV, n, pgmName)
+        *argV = (char **) cdf_AllocateMemory ((size_t)*argC * sizeof(char *), FatalError);
+	n = 0;
+	MAKEstrARGv (argV, n, pgmName)
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
-		CloseDialog (dialogP);
+	CloseDialog (dialogP);
         return TRUE;
       }
       /************************************************************************
@@ -1532,7 +1848,7 @@ char **argV[];
       case CANCELin:
 #ifdef __MWERKS__
         DisposeRoutineDescriptor (FilterDialogQOPsoUPP);
-		DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
+	DisposeRoutineDescriptor (OutlineDefaultButtonUPP);
 #endif
         CloseDialog (dialogP);
         return FALSE;
@@ -1540,22 +1856,22 @@ char **argV[];
       * Select CDF specification.
       ************************************************************************/
       case CDFSELECTin: {
-		StandardFileReply CDFreply;
-		char CDFpath[DU_MAX_PATH_LEN+1];
+	StandardFileReply CDFreply;
+	char CDFpath[DU_MAX_PATH_LEN+1];
 #ifndef __MWERKS__
-		StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
+	StandardGetFile (FilterForCDFs, -1, NULL, &CDFreply);
 #else
-		FilterForCDFsUPP = NewFileFilterProc((ProcPtr) FilterForCDFs);
-		StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
-		DisposeRoutineDescriptor (FilterForCDFsUPP);
+	FilterForCDFsUPP = NewFileFilterProc((ProcPtr) FilterForCDFs);
+	StandardGetFile (FilterForCDFsUPP, -1, NULL, &CDFreply);
+	DisposeRoutineDescriptor (FilterForCDFsUPP);
 #endif
-		if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
-		  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
-		  CDFtext[0] = strlen (CDFpath);
-		  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
-		  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
-		}
-		break;
+	if (CDFreply.sfGood && !CDFreply.sfIsFolder && !CDFreply.sfIsVolume) {
+	  BuildMacPath (&CDFreply.sfFile, CDFpath, TRUE);
+	  CDFtext[0] = strlen (CDFpath);
+	  strcpyX ((char *) &CDFtext[1], CDFpath, 255);
+	  SetIText ((Handle) controlHs[CDFTEXTin], CDFtext);
+	}
+	break;
       }
       /************************************************************************
       * Select skeleton CDF.
@@ -1563,17 +1879,17 @@ char **argV[];
       * iBeam (instead of returning it to what it was).
       ************************************************************************/
       case SKTSELECTin: {
-		StandardFileReply sktReply;
-		char sktPath[DU_MAX_PATH_LEN+1], prompt[] = "Enter skeleton table:";
-		StandardPutFile (CtoPstr(prompt), CtoPstr(""), &sktReply);
-		if (sktReply.sfGood && !sktReply.sfIsFolder && !sktReply.sfIsVolume) {
-		  BuildMacPath (&sktReply.sfFile, sktPath, TRUE);
-		  sktText[0] = strlen (sktPath);
-		  strcpyX ((char *) &sktText[1], sktPath, 255);
-		  SetIText ((Handle) controlHs[SKTTEXTin], sktText);
-		}
-		SetCursor (&(qd.arrow));
-		break;
+	StandardFileReply sktReply;
+	char sktPath[DU_MAX_PATH_LEN+1], prompt[] = "Enter skeleton table:";
+	StandardPutFile (CtoPstr(prompt), CtoPstr(""), &sktReply);
+	if (sktReply.sfGood && !sktReply.sfIsFolder && !sktReply.sfIsVolume) {
+	  BuildMacPath (&sktReply.sfFile, sktPath, TRUE);
+	  sktText[0] = strlen (sktPath);
+	  strcpyX ((char *) &sktText[1], sktPath, 255);
+	  SetIText ((Handle) controlHs[SKTTEXTin], sktText);
+	}
+	SetCursor (&(qd.arrow));
+	break;
       }
       /************************************************************************
       * Check boxes.
